"""GitHub 平台专用类型"""

from .enums import (
    GitHubAction,
    GitHubIssueState,
    GitHubMergeMethod,
    GitHubPostType,
    GitHubPRState,
    GitHubUserType,
)
from .models import (
    GitHubBranchRef,
    GitHubCommentInfo,
    GitHubCommit,
    GitHubForkee,
    GitHubIssueInfo,
    GitHubLabelInfo,
    GitHubMergeResult,
    GitHubPullRequestInfo,
    GitHubRelease,
    GitHubReleaseAsset,
    GitHubReleaseInfo,
    GitHubRepo,
    GitHubRepoInfo,
    GitHubUserInfo,
)
from .sender import GitHubSender
from .events import (
    GitHubEventData,
    GitHubForkEventData,
    GitHubIssueCommentEventData,
    GitHubIssueEventData,
    GitHubPREventData,
    GitHubPRReviewCommentEventData,
    GitHubPushEventData,
    GitHubReleaseEventData,
    GitHubStarEventData,
)

__all__ = [
    # enums
    "GitHubPostType",
    "GitHubAction",
    "GitHubIssueState",
    "GitHubPRState",
    "GitHubUserType",
    "GitHubMergeMethod",
    # event-embedded models
    "GitHubRepo",
    "GitHubCommit",
    "GitHubRelease",
    "GitHubReleaseAsset",
    "GitHubForkee",
    # API response models
    "GitHubUserInfo",
    "GitHubLabelInfo",
    "GitHubBranchRef",
    "GitHubIssueInfo",
    "GitHubCommentInfo",
    "GitHubPullRequestInfo",
    "GitHubMergeResult",
    "GitHubReleaseInfo",
    "GitHubRepoInfo",
    # sender
    "GitHubSender",
    # events
    "GitHubEventData",
    "GitHubIssueEventData",
    "GitHubIssueCommentEventData",
    "GitHubPREventData",
    "GitHubPRReviewCommentEventData",
    "GitHubPushEventData",
    "GitHubStarEventData",
    "GitHubForkEventData",
    "GitHubReleaseEventData",
]



from .base import IAPIClient
from .client import BotAPIClient
from .errors import (
    APIError,
    APINotFoundError,
    APIPermissionError,
    APIRequestError,
)
from .traits import IMessaging, IGroupManage, IQuery, IFileTransfer
from .qq import IQQAPIClient, QQAPIClient
from .bilibili import IBiliAPIClient
from .github import IGitHubAPIClient
from .misc import MiscAPI

__all__ = [
    "IAPIClient",
    "BotAPIClient",
    "APIError",
    "APIRequestError",
    "APIPermissionError",
    "APINotFoundError",
    # traits
    "IMessaging",
    "IGroupManage",
    "IQuery",
    "IFileTransfer",
    # qq
    "IQQAPIClient",
    "QQAPIClient",
    # bilibili
    "IBiliAPIClient",
    # github
    "IGitHubAPIClient",
    # misc
    "MiscAPI",
]



"""AI 适配器配置模型"""

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class AIConfig(BaseModel):
    """AI 适配器配置

    Parameters
    ----------
    api_key:
        API Key，为空时由 litellm 自动从环境变量读取
        （如 ``OPENAI_API_KEY``、``ANTHROPIC_API_KEY`` 等）。
    base_url:
        自定义 API 端点（兼容 DeepSeek / 通义千问 / Kimi 等国内 LLM）。
    completion_model:
        Chat Completion 默认模型（如 ``"gpt-4"``、``"deepseek-chat"``）。
    embedding_model:
        Embedding 默认模型（如 ``"text-embedding-3-small"``）。
    image_model:
        图像生成默认模型（如 ``"dall-e-3"``）。
    timeout:
        请求超时（秒）。
    max_tokens:
        默认最大 token 数，为 ``None`` 时由模型自行决定。
    """

    api_key: str = ""
    base_url: str = ""
    completion_model: str = ""
    embedding_model: str = ""
    image_model: str = ""
    timeout: float = 120.0
    max_tokens: Optional[int] = None



"""私信操作 API Mixin"""

from __future__ import annotations

from typing import Any


class SessionAPIMixin:
    async def send_private_msg(self, user_id: int, content: str) -> Any:
        from bilibili_api import session as bili_session

        return await bili_session.send_msg(
            self._credential,
            receiver_id=user_id,
            msg_type=bili_session.EventType.TEXT,
            content=content,
        )

    async def send_private_image(self, user_id: int, image_url: str) -> Any:
        from bilibili_api import session as bili_session

        return await bili_session.send_msg(
            self._credential,
            receiver_id=user_id,
            msg_type=bili_session.EventType.PICTURE,
            content=image_url,
        )

    async def get_session_history(self, user_id: int, count: int = 20) -> list:
        from bilibili_api import session as bili_session

        resp = await bili_session.fetch_session_msgs(
            self._credential,
            talker_id=user_id,
            session_type=1,
            begin_seqno=0,
        )
        messages = resp.get("messages", [])
        return messages[:count]



"""
登录认证模块

负责 NapCat WebUI 认证和 QQ 登录管理。
"""

import time
from enum import IntEnum
from typing import Optional, TYPE_CHECKING

import qrcode

from ncatbot.utils import get_log
from .webui_client import (
    WebUIClient,
    WebUIAuthError,
    WebUIConnectionError as _WebUIConnectionError,
)

if TYPE_CHECKING:
    from ncatbot.utils.config.models import NapCatConfig

LOG = get_log("AuthHandler")


class LoginStatus(IntEnum):
    OK = 0
    NOT_LOGGED_IN = 1
    UIN_MISMATCH = 2
    ABNORMAL = 3


class QQLoginedError(Exception):
    pass


class AuthError(Exception):
    pass


class WebUIConnectionError(AuthError):
    pass


class LoginTimeoutError(AuthError):
    pass


class AuthHandler:
    """登录认证处理器

    Parameters
    ----------
    napcat_config:
        NapCatConfig 实例。
    bot_uin:
        目标 QQ 号。
    """

    QRCODE_LOGIN_TIMEOUT = 60
    QRCODE_FETCH_TIMEOUT = 15

    def __init__(
        self,
        napcat_config: "NapCatConfig",
        bot_uin: str = "",
        webui_client: Optional[WebUIClient] = None,
    ):
        self._napcat_config = napcat_config
        self._bot_uin = bot_uin

        if webui_client is not None and webui_client.is_connected:
            self._client = webui_client
        else:
            self._client = WebUIClient(napcat_config)
            try:
                self._client.connect()
            except _WebUIConnectionError as e:
                raise WebUIConnectionError(str(e)) from e
            except WebUIAuthError as e:
                raise AuthError(str(e)) from e

    def _api_call(
        self,
        endpoint: str,
        payload: Optional[dict] = None,
        timeout: int = 5,
    ) -> dict:
        return self._client.api_call(endpoint, payload, timeout)

    # ==================== 登录状态检查 ====================

    def check_login_status(self) -> bool:
        try:
            data = self._api_call("/api/QQLogin/CheckLoginStatus")
            return data.get("data", {}).get("isLogin", False)
        except Exception:
            LOG.warning("检查登录状态超时, 默认未登录")
            return False

    def get_login_info(self) -> Optional[dict]:
        try:
            return self._api_call("/api/QQLogin/GetQQLoginInfo").get("data", {})
        except Exception:
            LOG.warning("获取登录信息超时")
            return None

    def report_status(self) -> LoginStatus:
        if not self.check_login_status():
            return LoginStatus.NOT_LOGGED_IN

        info = self.get_login_info()
        if not info:
            return LoginStatus.ABNORMAL

        target_uin = self._bot_uin
        current_uin = str(info.get("uin", ""))
        is_online = info.get("online", False)

        if current_uin != target_uin:
            return LoginStatus.UIN_MISMATCH
        if not is_online:
            return LoginStatus.ABNORMAL

        return LoginStatus.OK

    # ==================== 快速登录 ====================

    def get_quick_login_list(self) -> list:
        try:
            data = self._api_call("/api/QQLogin/GetQuickLoginListNew")
            records = data.get("data", [])
            return [str(r["uin"]) for r in records if r.get("isQuickLogin")]
        except Exception:
            LOG.warning("获取快速登录列表失败")
            return []

    def quick_login(self) -> bool:
        uin = self._bot_uin
        quick_list = self.get_quick_login_list()
        LOG.info(f"快速登录列表: {quick_list}")

        if uin not in quick_list:
            return False

        LOG.info("正在发送快速登录请求...")
        try:
            status = self._api_call(
                "/api/QQLogin/SetQuickLogin",
                payload={"uin": uin},
                timeout=8,
            )
            success = status.get("message", "") in ["success", "QQ Is Logined"]
            if not success:
                LOG.warning(f"快速登录请求失败: {status}")
            return success
        except Exception:
            LOG.warning("快速登录失败")
            return False

    # ==================== 二维码登录 ====================

    def get_qrcode_url(self) -> str:
        expire_time = time.time() + self.QRCODE_FETCH_TIMEOUT

        while time.time() < expire_time:
            time.sleep(0.2)
            try:
                data = self._api_call("/api/QQLogin/GetQQLoginQrcode")
                if data.get("message") == "QQ Is Logined":
                    raise QQLoginedError()

                qrcode_url = data.get("data", {}).get("qrcode")
                if qrcode_url:
                    return qrcode_url
            except QQLoginedError:
                raise
            except Exception:
                pass

        raise LoginTimeoutError("获取二维码超时")

    @staticmethod
    def show_qrcode(url: str) -> None:
        qr = qrcode.QRCode()
        qr.add_data(url)
        qr.print_ascii(invert=True)

    def qrcode_login(self) -> None:
        try:
            try:
                qrcode_url = self.get_qrcode_url()
                self.show_qrcode(qrcode_url)
            except QQLoginedError:
                if self.check_login_status():
                    LOG.info("QQ 已登录（账号验证由 launcher 处理）")
                    return
                LOG.error("NapCat 报告 QQ 已登录但状态检查不一致, 请物理重启本机")
                raise AuthError("登录状态异常")

            expire_time = time.time() + self.QRCODE_LOGIN_TIMEOUT
            warn_time = time.time() + 30

            while not self.check_login_status():
                if time.time() > expire_time:
                    LOG.error("登录超时, 请重新操作")
                    raise LoginTimeoutError("登录超时")
                if time.time() > warn_time:
                    LOG.warning("二维码即将失效, 请尽快扫码登录")
                    warn_time += 60

            LOG.info("扫码登录成功")

        except (QQLoginedError, LoginTimeoutError, AuthError):
            raise
        except Exception as e:
            LOG.error(f"二维码登录出错: {e}")
            raise AuthError("登录失败")

    # ==================== 主登录流程 ====================

    def login(self) -> None:
        """通过 WebUI 执行登录 (快速登录 → 二维码)。

        仅在 launcher 确认 WS 不通 (即未登录) 时调用。
        """
        if self.quick_login():
            LOG.info("快速登录成功")
            return

        self.qrcode_login()



"""插件入口。"""

from ncatbot.plugin import NcatBotPlugin


class PluginClassName(NcatBotPlugin):
    """PluginName 插件。

    注意：插件元数据由 manifest.toml 提供，不要在类中声明。
    """

    async def on_load(self):
        self.logger.info(f"{self.name} 已加载")

    async def on_close(self):
        self.logger.info(f"{self.name} 已卸载")



"""IGitHubAPIClient — GitHub 平台 API 接口

定义 GitHub 适配器所有可用 API 方法。
由 GitHubBotAPI 实现。
"""

from __future__ import annotations

from abc import abstractmethod
from typing import List, Optional, Union

from ncatbot.api.base import IAPIClient
from ncatbot.types.github.enums import GitHubMergeMethod
from ncatbot.types.github.models import (
    GitHubCommentInfo,
    GitHubIssueInfo,
    GitHubLabelInfo,
    GitHubMergeResult,
    GitHubPullRequestInfo,
    GitHubReleaseAsset,
    GitHubReleaseInfo,
    GitHubRepoInfo,
    GitHubUserInfo,
)


class IGitHubAPIClient(IAPIClient):
    """GitHub 平台 Bot API 接口"""

    # ---- Issue 操作 ----

    @abstractmethod
    async def create_issue(
        self,
        repo: str,
        title: str,
        body: str = "",
        labels: Optional[List[str]] = None,
        assignees: Optional[List[str]] = None,
    ) -> GitHubIssueInfo:
        """创建 Issue"""

    @abstractmethod
    async def update_issue(
        self,
        repo: str,
        issue_number: int,
        *,
        title: Optional[str] = None,
        body: Optional[str] = None,
        state: Optional[str] = None,
        labels: Optional[List[str]] = None,
        assignees: Optional[List[str]] = None,
    ) -> GitHubIssueInfo:
        """更新 Issue"""

    @abstractmethod
    async def close_issue(self, repo: str, issue_number: int) -> GitHubIssueInfo:
        """关闭 Issue"""

    @abstractmethod
    async def reopen_issue(self, repo: str, issue_number: int) -> GitHubIssueInfo:
        """重新打开 Issue"""

    @abstractmethod
    async def get_issue(self, repo: str, issue_number: int) -> GitHubIssueInfo:
        """获取 Issue 详情"""

    @abstractmethod
    async def add_labels(
        self, repo: str, issue_number: int, labels: List[str]
    ) -> List[GitHubLabelInfo]:
        """添加标签"""

    @abstractmethod
    async def remove_label(self, repo: str, issue_number: int, label: str) -> None:
        """移除标签"""

    @abstractmethod
    async def set_assignees(
        self, repo: str, issue_number: int, assignees: List[str]
    ) -> GitHubIssueInfo:
        """设置负责人"""

    # ---- 评论操作 ----

    @abstractmethod
    async def create_issue_comment(
        self, repo: str, issue_number: int, body: str
    ) -> GitHubCommentInfo:
        """创建 Issue/PR 评论"""

    @abstractmethod
    async def update_comment(
        self, repo: str, comment_id: int, body: str
    ) -> GitHubCommentInfo:
        """更新评论"""

    @abstractmethod
    async def delete_comment(self, repo: str, comment_id: int) -> None:
        """删除评论"""

    @abstractmethod
    async def list_issue_comments(
        self, repo: str, issue_number: int, page: int = 1, per_page: int = 30
    ) -> List[GitHubCommentInfo]:
        """列出评论"""

    # ---- PR 操作 ----

    @abstractmethod
    async def create_pr_comment(
        self, repo: str, pr_number: int, body: str
    ) -> GitHubCommentInfo:
        """创建 PR 评论 (等同 create_issue_comment)"""

    @abstractmethod
    async def merge_pr(
        self,
        repo: str,
        pr_number: int,
        *,
        merge_method: Union[GitHubMergeMethod, str] = GitHubMergeMethod.MERGE,
        commit_title: Optional[str] = None,
        commit_message: Optional[str] = None,
    ) -> GitHubMergeResult:
        """合并 PR"""

    @abstractmethod
    async def close_pr(self, repo: str, pr_number: int) -> GitHubPullRequestInfo:
        """关闭 PR"""

    @abstractmethod
    async def request_review(
        self, repo: str, pr_number: int, reviewers: List[str]
    ) -> GitHubPullRequestInfo:
        """请求 Review"""

    @abstractmethod
    async def get_pr(self, repo: str, pr_number: int) -> GitHubPullRequestInfo:
        """获取 PR 详情"""

    # ---- 查询 ----

    @abstractmethod
    async def get_repo(self, repo: str) -> GitHubRepoInfo:
        """获取仓库信息"""

    @abstractmethod
    async def get_user(self, username: str) -> GitHubUserInfo:
        """获取用户信息"""

    @abstractmethod
    async def get_authenticated_user(self) -> GitHubUserInfo:
        """获取当前认证用户"""

    # ---- Release 操作 ----

    @abstractmethod
    async def get_release(
        self, repo: str, release_id: Union[str, int]
    ) -> GitHubReleaseInfo:
        """获取指定 Release 详情"""

    @abstractmethod
    async def get_release_by_tag(self, repo: str, tag: str) -> GitHubReleaseInfo:
        """按 Tag 获取 Release 详情"""

    @abstractmethod
    async def get_latest_release(self, repo: str) -> GitHubReleaseInfo:
        """获取最新 Release"""

    @abstractmethod
    async def list_release_assets(
        self, repo: str, release_id: Union[str, int]
    ) -> List[GitHubReleaseAsset]:
        """列出 Release 的所有 Assets"""



"""Bilibili 平台数据模型 — 非事件类结构化数据"""

from __future__ import annotations

from typing import Optional, Tuple

from pydantic import BaseModel, Field


__all__ = [
    "LiveRoomInfo",
    "RoomInfo",
    "AnchorInfo",
    "WatchedShow",
    "DynamicStatInfo",
    "DynamicVideoInfo",
    "DynamicMusicInfo",
    "DynamicArticleInfo",
    "DynamicLiveRcmdInfo",
    "VideoOwner",
    "VideoStat",
    "VideoPage",
    "VideoStaffMember",
    "VideoInfo",
    "ParsedBiliId",
]


class RoomInfo(BaseModel):
    """直播间信息"""

    uid: int = 0
    room_id: int = 0
    title: str = ""
    cover_url: str = ""
    background_url: str = ""
    description: str = ""
    tags: Tuple[str, ...] = ()
    live_status: int = 0
    live_start_time: int = 0
    parent_area_name: str = ""
    parent_area_id: int = 0
    area_name: str = ""
    area_id: int = 0
    keyframe_url: str = ""
    online: int = 0


class AnchorInfo(BaseModel):
    """主播信息"""

    name: str = ""
    face_url: str = ""
    gender: str = ""
    official_info: str = ""
    fanclub_name: str = ""
    fanclub_num: int = 0
    live_level: int = 0
    live_score: int = 0
    live_upgrade_score: int = 0


class WatchedShow(BaseModel):
    """观看榜信息"""

    num: int = 0
    text_small: str = ""
    text_large: str = ""


class LiveRoomInfo(BaseModel):
    """直播间完整信息"""

    room_info: RoomInfo = Field(default_factory=RoomInfo)
    anchor_info: AnchorInfo = Field(default_factory=AnchorInfo)
    watched_show: WatchedShow = Field(default_factory=WatchedShow)

    @classmethod
    def from_raw(cls, data: dict) -> Optional["LiveRoomInfo"]:
        """从 bilibili-api ``LiveRoom.get_room_info()`` 原始数据构造模型。"""
        try:
            ri = data.get("room_info", {})
            tags_str: str = ri.get("tags", "")
            tags = tuple(tags_str.split(",")) if tags_str else ()

            room_info = RoomInfo(
                uid=ri.get("uid", 0),
                room_id=ri.get("room_id", 0),
                title=ri.get("title", ""),
                cover_url=ri.get("cover", ""),
                background_url=ri.get("background", ""),
                description=ri.get("description", ""),
                tags=tags,
                live_status=ri.get("live_status", 0),
                live_start_time=ri.get("live_start_time", 0),
                parent_area_name=ri.get("parent_area_name", ""),
                parent_area_id=ri.get("parent_area_id", 0),
                area_name=ri.get("area_name", ""),
                area_id=ri.get("area_id", 0),
                keyframe_url=ri.get("keyframe", ""),
                online=ri.get("online", 0),
            )

            ai = data.get("anchor_info", {})
            base = ai.get("base_info", {})
            medal = ai.get("medal_info", {})
            live_info = ai.get("live_info", {})
            official = base.get("official_info", {})

            anchor_info = AnchorInfo(
                name=base.get("uname", ""),
                face_url=base.get("face", ""),
                gender=base.get("gender", ""),
                official_info=official.get("title", ""),
                fanclub_name=medal.get("medal_name", ""),
                fanclub_num=medal.get("fansclub", 0),
                live_level=live_info.get("level", 0),
                live_score=live_info.get("score", 0),
                live_upgrade_score=live_info.get("upgrade_score", 0),
            )

            ws = data.get("watched_show", {})
            watched_show = WatchedShow(
                num=ws.get("num", 0),
                text_small=ws.get("text_small", ""),
                text_large=ws.get("text_large", ""),
            )

            return cls(
                room_info=room_info,
                anchor_info=anchor_info,
                watched_show=watched_show,
            )
        except Exception:
            return None


# ==================== 动态相关模型 ====================


class DynamicStatInfo(BaseModel):
    """动态统计信息"""

    comment_count: int = 0
    like_count: int = 0
    forward_count: int = 0


class DynamicVideoInfo(BaseModel):
    """动态中的视频信息"""

    av_id: str = ""
    bv_id: str = ""
    title: str = ""
    cover: str = ""
    desc: str = ""
    duration_text: str = ""
    play_count: str = ""
    danmaku_count: str = ""
    dynamic_text: str = ""


class DynamicMusicInfo(BaseModel):
    """动态中的音乐信息"""

    music_id: str = ""
    title: str = ""
    cover: str = ""
    label: str = ""
    dynamic_text: str = ""


class DynamicArticleInfo(BaseModel):
    """动态中的专栏信息"""

    title: str = ""
    summary: str = ""
    has_more: bool = False
    article_id: int = 0


class DynamicLiveRcmdInfo(BaseModel):
    """动态中的直播推荐信息"""

    room_id: int = 0
    live_status: int = 0
    title: str = ""
    cover: str = ""
    online: int = 0
    area_id: int = 0
    area_name: str = ""
    parent_area_id: int = 0
    parent_area_name: str = ""
    live_start_time: int = 0


# ==================== 视频信息模型 ====================


class VideoOwner(BaseModel):
    """视频 UP 主信息"""

    mid: int = 0
    name: str = ""
    face: str = ""


class VideoStat(BaseModel):
    """视频统计信息"""

    view: int = 0
    danmaku: int = 0
    reply: int = 0
    favorite: int = 0
    coin: int = 0
    share: int = 0
    like: int = 0
    dislike: int = 0
    his_rank: int = 0
    now_rank: int = 0


class VideoPage(BaseModel):
    """视频分 P 信息"""

    cid: int = 0
    page: int = 0
    part: str = ""
    duration: int = 0


class VideoStaffMember(BaseModel):
    """联合投稿成员"""

    mid: int = 0
    name: str = ""
    title: str = ""
    face: str = ""
    follower: int = 0


class VideoInfo(BaseModel):
    """视频完整信息"""

    bvid: str = ""
    aid: int = 0
    title: str = ""
    pic: str = ""
    desc: str = ""
    pubdate: int = 0
    ctime: int = 0
    duration: int = 0
    videos: int = 1
    tid: int = 0
    tname: str = ""
    copyright: int = 0
    state: int = 0
    owner: VideoOwner = Field(default_factory=VideoOwner)
    stat: VideoStat = Field(default_factory=VideoStat)
    pages: Tuple[VideoPage, ...] = ()
    staff: Tuple[VideoStaffMember, ...] = ()
    dynamic: str = ""
    cid: int = 0
    season_id: Optional[int] = None

    @property
    def duration_text(self) -> str:
        """格式化时长 HH:MM:SS 或 MM:SS"""
        m, s = divmod(self.duration, 60)
        h, m = divmod(m, 60)
        return f"{h:02d}:{m:02d}:{s:02d}" if h else f"{m:02d}:{s:02d}"

    @property
    def url(self) -> str:
        return f"https://www.bilibili.com/video/{self.bvid}"

    @classmethod
    def from_raw(cls, data: dict) -> Optional["VideoInfo"]:
        """从 bilibili-api ``video.Video.get_info()`` 原始数据构造模型。"""
        try:
            owner_raw = data.get("owner", {})
            owner = VideoOwner(
                mid=owner_raw.get("mid", 0),
                name=owner_raw.get("name", ""),
                face=owner_raw.get("face", ""),
            )

            stat_raw = data.get("stat", {})
            stat = VideoStat(
                view=stat_raw.get("view", 0),
                danmaku=stat_raw.get("danmaku", 0),
                reply=stat_raw.get("reply", 0),
                favorite=stat_raw.get("favorite", 0),
                coin=stat_raw.get("coin", 0),
                share=stat_raw.get("share", 0),
                like=stat_raw.get("like", 0),
                dislike=stat_raw.get("dislike", 0),
                his_rank=stat_raw.get("his_rank", 0),
                now_rank=stat_raw.get("now_rank", 0),
            )

            pages = tuple(
                VideoPage(
                    cid=p.get("cid", 0),
                    page=p.get("page", 0),
                    part=p.get("part", ""),
                    duration=p.get("duration", 0),
                )
                for p in data.get("pages", [])
            )

            staff = tuple(
                VideoStaffMember(
                    mid=s.get("mid", 0),
                    name=s.get("name", ""),
                    title=s.get("title", ""),
                    face=s.get("face", ""),
                    follower=s.get("follower", 0),
                )
                for s in data.get("staff", [])
            )

            return cls(
                bvid=data.get("bvid", ""),
                aid=data.get("aid", 0),
                title=data.get("title", ""),
                pic=data.get("pic", ""),
                desc=data.get("desc", ""),
                pubdate=data.get("pubdate", 0),
                ctime=data.get("ctime", 0),
                duration=data.get("duration", 0),
                videos=data.get("videos", 1),
                tid=data.get("tid", 0),
                tname=data.get("tname_v2", "") or data.get("tname", ""),
                copyright=data.get("copyright", 0),
                state=data.get("state", 0),
                owner=owner,
                stat=stat,
                pages=pages,
                staff=staff,
                dynamic=data.get("dynamic", ""),
                cid=data.get("cid", 0),
                season_id=data.get("season_id"),
            )
        except Exception:
            return None


# ==================== B站链接解析结果 ====================


class ParsedBiliId(BaseModel):
    """B站视频 ID 解析结果"""

    id_type: str = ""  # "bv" | "av"
    bvid: str = ""
    aid: int = 0
    raw_url: str = ""
    redirected_url: str = ""

    @property
    def video_id(self) -> str:
        """可直接传入 ``get_video_info`` 的视频 ID 字符串。"""
        return self.bvid if self.id_type == "bv" else f"av{self.aid}"



"""config 命令组 — 配置管理。"""

import click
import yaml

from ..utils.colors import success, error, key, value, header, warning


def _get_manager():
    from ncatbot.utils import get_config_manager

    return get_config_manager()


def _resolve_value(raw: str):
    """将 CLI 传入的字符串转换为合适的 Python 类型。"""
    if raw.lower() in ("true", "yes"):
        return True
    if raw.lower() in ("false", "no"):
        return False
    try:
        return int(raw)
    except ValueError:
        pass
    # 尝试解析为列表 (JSON 格式)
    if raw.startswith("["):
        import json

        try:
            return json.loads(raw)
        except json.JSONDecodeError:
            pass
    return raw


@click.group()
def config():
    """配置管理"""


@config.command()
def show():
    """显示当前配置"""
    mgr = _get_manager()
    data = mgr.config.to_dict()
    click.echo(header("当前配置:"))
    click.echo(
        yaml.dump(data, allow_unicode=True, default_flow_style=False, sort_keys=False)
    )


@config.command("get")
@click.argument("config_key")
def config_get(config_key: str):
    """读取配置项（支持点分路径，如 napcat.ws_uri）"""
    mgr = _get_manager()
    cfg = mgr.config
    full_key = cfg.get_field_paths().get(config_key, config_key)
    parts = full_key.split(".")
    obj = cfg
    try:
        for part in parts:
            obj = getattr(obj, part)
    except AttributeError:
        click.echo(error(f"未知的配置项: {config_key}"))
        raise SystemExit(1)
    click.echo(f"{key(config_key)} = {value(str(obj))}")


@config.command("set")
@click.argument("config_key")
@click.argument("config_value")
def config_set(config_key: str, config_value: str):
    """设置配置项（支持点分路径）"""
    mgr = _get_manager()
    converted = _resolve_value(config_value)
    try:
        mgr.update_value(config_key, converted)
        mgr.save()
    except Exception as e:
        click.echo(error(f"设置失败: {e}"))
        raise SystemExit(1)
    click.echo(success(f"已设置 {config_key} = {converted}"))


@config.command()
def check():
    """检查配置问题（安全性 + 必填项）"""
    mgr = _get_manager()
    issues = mgr.get_issues()
    if not issues:
        click.echo(success("配置检查通过，无问题。"))
        return
    click.echo(warning(f"发现 {len(issues)} 个问题:"))
    for i, issue in enumerate(issues, 1):
        click.echo(f"  {i}. {issue}")



"""IAIAPIClient — AI 平台 API 接口

定义 AI 适配器所有可用 API 方法。
由 AIBotAPI 实现。
"""

from __future__ import annotations

from abc import abstractmethod
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union

from ncatbot.api.base import IAPIClient

if TYPE_CHECKING:
    from ncatbot.types import Image as ImageSegment
    from ncatbot.types.common.segment.array import MessageArray
    from ncatbot.types.common.segment.base import MessageSegment

ChatInput = Union[str, List[dict], "MessageArray", "MessageSegment"]


class IAIAPIClient(IAPIClient):
    """AI 平台 Bot API 接口"""

    # ---- Chat Completion ----

    @abstractmethod
    async def chat(
        self,
        content_or_messages: ChatInput,
        *,
        model: Optional[str] = None,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        nickname_map: Optional[Dict[str, str]] = None,
        **kwargs: Any,
    ) -> Any:
        """Chat Completion

        Parameters
        ----------
        content_or_messages:
            - ``str``: 自动包装为 ``[{"role": "user", "content": str}]``
            - ``list[dict]``: 直接作为 messages 参数
            - ``MessageArray``: 自动转为多模态 content（文本 + 图片）
            - ``MessageSegment``: 单个消息段（Image / PlainText 等）
        model:
            覆盖 ``completion_model``。
        temperature:
            采样温度。
        max_tokens:
            最大生成 token 数。
        nickname_map:
            ``{user_id: 昵称}`` 映射，At 段转为可读文本。

        Returns
        -------
        litellm.ModelResponse
        """

    # ---- Embeddings ----

    @abstractmethod
    async def embeddings(
        self,
        input_text: Union[str, List[str]],
        *,
        model: Optional[str] = None,
        **kwargs: Any,
    ) -> Any:
        """文本向量化

        Parameters
        ----------
        input_text:
            待嵌入的文本或文本列表。
        model:
            覆盖 ``embedding_model``。

        Returns
        -------
        litellm.EmbeddingResponse
        """

    # ---- Image Generation ----

    @abstractmethod
    async def image_generation(
        self,
        prompt: str,
        *,
        model: Optional[str] = None,
        size: Optional[str] = None,
        **kwargs: Any,
    ) -> Any:
        """图像生成

        Parameters
        ----------
        prompt:
            图像描述文本。
        model:
            覆盖 ``image_model``。
        size:
            图像尺寸（如 ``"1024x1024"``）。

        Returns
        -------
        litellm.ImageResponse
        """

    # ---- Sugar 便捷方法 ----

    @abstractmethod
    async def chat_text(
        self,
        content_or_messages: ChatInput,
        *,
        model: Optional[str] = None,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        nickname_map: Optional[Dict[str, str]] = None,
        **kwargs: Any,
    ) -> str:
        """Chat Completion — 直接返回文本

        与 ``chat()`` 参数相同，返回 ``choices[0].message.content``。
        """

    @abstractmethod
    async def generate_image(
        self,
        prompt: str,
        *,
        model: Optional[str] = None,
        size: Optional[str] = None,
        **kwargs: Any,
    ) -> "ImageSegment":
        """图像生成 — 直接返回 Image 消息段

        与 ``image_generation()`` 参数相同，返回 ``ncatbot.types.Image``。
        """



"""
异步事件分发器

纯事件路由：通过 callback 接收事件，广播到所有事件流消费者。
外部通过 events() / async for / wait_event 消费事件。
"""

from __future__ import annotations

import asyncio
from typing import (
    Awaitable,
    Callable,
    Optional,
    Set,
    Union,
    TYPE_CHECKING,
)

from ncatbot.types.qq import EventType
from ncatbot.utils import get_log

from .event import Event
from .stream import EventStream, _STOP

if TYPE_CHECKING:
    from ncatbot.types import BaseEventData

LOG = get_log("AsyncEventDispatcher")


class AsyncEventDispatcher:
    """异步事件分发器

    纯事件路由器：
    - 生产端：外部通过 :meth:`callback` 获取回调函数，将 ``BaseEventData`` 喂入。
    - 消费端：通过 :meth:`events` / ``async for`` / :meth:`wait_event` 消费事件。

    用法::

        dispatcher = AsyncEventDispatcher()

        # 将 callback 交给 adapter
        adapter.set_event_callback(dispatcher.callback)

        # 消费事件
        async with dispatcher.events(EventType.MESSAGE) as stream:
            async for event in stream:
                print(event.type, event.data)
    """

    _DEFAULT_STREAM_QUEUE_SIZE = 500

    def __init__(
        self,
        stream_queue_size: int = _DEFAULT_STREAM_QUEUE_SIZE,
    ):
        self._stream_queues: Set[asyncio.Queue] = set()
        self._stream_queue_size = stream_queue_size
        self._waiters: list[_Waiter] = []
        self._closed = False

    # ---- 生产端 ----

    @property
    def callback(self) -> Callable[["BaseEventData"], "Awaitable[None]"]:
        """返回一个异步回调函数，供外部（如 Adapter）推送事件。

        回调签名: ``async (data: BaseEventData) -> None``
        """
        return self._on_event

    async def _on_event(self, data: "BaseEventData") -> None:
        """接收外部推送的事件数据，resolve 类型后广播。"""
        if self._closed:
            return

        event_type = data.resolve_type()
        platform = getattr(data, "platform", "unknown")
        event = Event(type=event_type, data=data, platform=platform)

        self._broadcast(event)
        self._resolve_waiters(event)

    # ---- 消费端：事件流 ----

    def events(
        self,
        event_type: Optional[Union[str, EventType]] = None,
    ) -> EventStream:
        """创建事件流。

        Args:
            event_type: 按类型过滤。
                - ``EventType`` 枚举: 前缀匹配（如 ``EventType.MESSAGE`` 匹配所有 message 子类型）。
                - ``str``: 前缀匹配（如 ``"message.group"`` 精确匹配，``"message"`` 匹配所有）。
                - ``None``: 不过滤，接收全部事件。

        Returns:
            :class:`EventStream` 异步可迭代对象，支持 ``async with`` / ``async for`` / ``aclose()``。
        """
        if self._closed:
            raise RuntimeError("AsyncEventDispatcher 已关闭")

        queue: asyncio.Queue = asyncio.Queue(maxsize=self._stream_queue_size)
        self._stream_queues.add(queue)
        return EventStream(self, queue, event_type)

    def __aiter__(self):
        return self.events()

    # ---- 消费端：一次性等待 ----

    async def wait_event(
        self,
        predicate: Optional[Callable[[Event], bool]] = None,
        timeout: Optional[float] = None,
    ) -> Event:
        """等待下一个满足条件的事件。

        Args:
            predicate: 过滤函数，``None`` 表示接受任意事件。
            timeout: 超时秒数，``None`` 表示无限等待。

        Returns:
            匹配的 :class:`Event` 实例。

        Raises:
            asyncio.TimeoutError: 超时未等到匹配事件。
            RuntimeError: dispatcher 已关闭。
        """
        if self._closed:
            raise RuntimeError("AsyncEventDispatcher 已关闭")

        loop = asyncio.get_running_loop()
        future: asyncio.Future[Event] = loop.create_future()
        waiter = _Waiter(predicate=predicate, future=future)
        self._waiters.append(waiter)

        try:
            if timeout is None:
                return await future
            return await asyncio.wait_for(future, timeout)
        finally:
            try:
                self._waiters.remove(waiter)
            except ValueError:
                pass

    # ---- 内部方法 ----

    def _unregister_stream(self, queue: asyncio.Queue) -> None:
        """由 EventStream.aclose() 调用，注销队列。"""
        self._stream_queues.discard(queue)

    def _broadcast(self, event: Event) -> None:
        """将事件广播到所有活跃的流队列。"""
        for queue in tuple(self._stream_queues):
            if queue.full():
                try:
                    queue.get_nowait()
                except asyncio.QueueEmpty:
                    pass
                LOG.debug("事件流队列已满，丢弃最旧事件")
            try:
                queue.put_nowait(event)
            except Exception:
                LOG.exception("写入事件流队列失败")

    def _resolve_waiters(self, event: Event) -> None:
        """尝试 resolve 所有匹配的一次性 waiter。"""
        for waiter in tuple(self._waiters):
            if waiter.future.done():
                continue
            try:
                if waiter.predicate is None or waiter.predicate(event):
                    waiter.future.set_result(event)
            except Exception as exc:
                waiter.future.set_exception(exc)

    async def close(self) -> None:
        """关闭 dispatcher，终止所有活跃事件流和 waiter。"""
        if self._closed:
            return
        self._closed = True

        # 终止所有 stream
        for queue in tuple(self._stream_queues):
            try:
                queue.put_nowait(_STOP)
            except Exception:
                pass
        self._stream_queues.clear()

        # 终止所有 waiter
        closed_err = RuntimeError("AsyncEventDispatcher 已关闭")
        for waiter in self._waiters:
            if not waiter.future.done():
                waiter.future.set_exception(closed_err)
        self._waiters.clear()


class _Waiter:
    """wait_event() 的内部状态。"""

    __slots__ = ("predicate", "future")

    def __init__(
        self,
        predicate: Optional[Callable[[Event], bool]],
        future: asyncio.Future[Event],
    ):
        self.predicate = predicate
        self.future = future



"""GitHub 平台事件工厂"""

from __future__ import annotations

from typing import TYPE_CHECKING, Dict, Optional, Type

from ncatbot.types.common.base import BaseEventData
from ncatbot.types.github.enums import GitHubPostType
from ncatbot.types.github.events import (
    GitHubIssueEventData,
    GitHubIssueCommentEventData,
    GitHubPREventData,
    GitHubPRReviewCommentEventData,
    GitHubPushEventData,
    GitHubStarEventData,
    GitHubForkEventData,
    GitHubReleaseEventData,
)
from ncatbot.event.common.base import BaseEvent
from .issue import GitHubIssueEvent, GitHubIssueCommentEvent
from .pr import GitHubPREvent, GitHubPRReviewCommentEvent
from .push import GitHubPushEvent
from .misc import GitHubStarEvent, GitHubForkEvent, GitHubReleaseEvent

if TYPE_CHECKING:
    from ncatbot.api import IAPIClient

__all__ = [
    "create_github_entity",
]

# 精确映射：数据模型类 → 实体类
_GITHUB_ENTITY_MAP: Dict[Type[BaseEventData], Type[BaseEvent]] = {
    GitHubIssueEventData: GitHubIssueEvent,
    GitHubIssueCommentEventData: GitHubIssueCommentEvent,
    GitHubPREventData: GitHubPREvent,
    GitHubPRReviewCommentEventData: GitHubPRReviewCommentEvent,
    GitHubPushEventData: GitHubPushEvent,
    GitHubStarEventData: GitHubStarEvent,
    GitHubForkEventData: GitHubForkEvent,
    GitHubReleaseEventData: GitHubReleaseEvent,
}

# post_type → 降级实体基类
_GITHUB_FALLBACK_MAP: Dict[str, Type[BaseEvent]] = {
    GitHubPostType.ISSUE: BaseEvent,
    GitHubPostType.PULL_REQUEST: BaseEvent,
    GitHubPostType.COMMENT: BaseEvent,
    GitHubPostType.PUSH: BaseEvent,
    GitHubPostType.STAR: BaseEvent,
    GitHubPostType.FORK: BaseEvent,
    GitHubPostType.RELEASE: BaseEvent,
}


def create_github_entity(data: BaseEventData, api: "IAPIClient") -> Optional[BaseEvent]:
    """GitHub 平台事件工厂"""
    entity_cls = _GITHUB_ENTITY_MAP.get(type(data))
    if entity_cls is None:
        entity_cls = _GITHUB_FALLBACK_MAP.get(data.post_type)
    if entity_cls is None:
        return None
    return entity_cls(data, api)



"""Bilibili 适配器配置模型"""

from __future__ import annotations

from typing import List

from pydantic import BaseModel, Field


class CommentWatch(BaseModel):
    """评论监听条目"""

    id: str
    type: str = "video"


class DynamicWatch(BaseModel):
    """动态监听条目"""

    uid: int


class BilibiliConfig(BaseModel):
    """Bilibili 适配器专属配置"""

    # 认证
    sessdata: str = ""
    bili_jct: str = ""
    buvid3: str = ""
    dedeuserid: str = ""
    ac_time_value: str = ""

    # 初始监听源
    live_rooms: List[int] = Field(default_factory=list)
    enable_session: bool = False
    comment_watches: List[CommentWatch] = Field(default_factory=list)
    dynamic_watches: List[DynamicWatch] = Field(default_factory=list)
    dynamic_page_watches: List[DynamicWatch] = Field(default_factory=list)

    # 轮询配置
    session_poll_interval: float = 6.0
    comment_poll_interval: float = 30.0
    dynamic_poll_interval: float = 600.0
    dynamic_page_poll_interval: float = 180.0

    # 连接
    max_retry: int = 5
    retry_after: float = 1.0



from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel, model_validator

__all__ = [
    "BaseSender",
]


class BaseSender(BaseModel):
    user_id: Optional[str] = None
    nickname: Optional[str] = None

    @model_validator(mode="before")
    @classmethod
    def _coerce_ids(cls, data: Any) -> Any:
        if isinstance(data, dict):
            for key, value in data.items():
                if key.endswith("_id") and isinstance(value, (int, float)):
                    data[key] = str(int(value))
        return data



"""Star / Fork / Release 事件实体"""

from __future__ import annotations

from typing import List

from .base import GitHubBaseEvent
from ..common.mixins import HasAttachments
from ncatbot.types.common import AttachmentList, Attachment, FileAttachment
from ncatbot.types.github import (
    GitHubStarEventData,
    GitHubForkEventData,
    GitHubReleaseEventData,
)
from ncatbot.types.github import (
    GitHubForkee,
    GitHubRelease,
    GitHubReleaseAsset,
    GitHubReleaseInfo,
)

__all__ = [
    "GitHubStarEvent",
    "GitHubForkEvent",
    "GitHubReleaseEvent",
]


class GitHubStarEvent(GitHubBaseEvent):
    """GitHub Star 事件"""

    _data: "GitHubStarEventData"

    @property
    def starred_at(self) -> str:
        return self._data.starred_at


class GitHubForkEvent(GitHubBaseEvent):
    """GitHub Fork 事件"""

    _data: "GitHubForkEventData"

    @property
    def forkee(self) -> "GitHubForkee":
        return self._data.forkee

    @property
    def forkee_full_name(self) -> str:
        return self._data.forkee.full_name


class GitHubReleaseEvent(GitHubBaseEvent, HasAttachments):
    """GitHub Release 事件"""

    _data: "GitHubReleaseEventData"

    @property
    def release(self) -> "GitHubRelease":
        return self._data.release

    @property
    def release_tag(self) -> str:
        return self._data.release.tag_name

    @property
    def release_name(self) -> str:
        return self._data.release.name

    @property
    def release_body(self) -> str:
        return self._data.release.body

    @property
    def prerelease(self) -> bool:
        return self._data.release.prerelease

    async def get_assets(self) -> "List[GitHubReleaseAsset]":
        """获取该 Release 的所有 Assets"""
        return await self._api.list_release_assets(
            repo=self._data.repo.full_name,
            release_id=self._data.release.id,
        )

    async def get_full_release(self) -> "GitHubReleaseInfo":
        """通过 GitHub API 获取该 Release 的完整信息（含 assets 等）"""
        return await self._api.get_release_by_tag(
            repo=self._data.repo.full_name,
            tag=self._data.release.tag_name,
        )

    async def get_attachments(self) -> "AttachmentList[Attachment]":
        """将 Release Assets 转换为跨平台 Attachment 列表"""
        from ncatbot.types.common.attachment import (
            AudioAttachment,
            ImageAttachment,
            VideoAttachment,
        )

        _CONTENT_TYPE_MAP: dict[str, type[Attachment]] = {
            "image": ImageAttachment,
            "video": VideoAttachment,
            "audio": AudioAttachment,
        }

        def _pick_cls(ct: str | None) -> type[Attachment]:
            if ct:
                major = ct.split("/", 1)[0]
                if major in _CONTENT_TYPE_MAP:
                    return _CONTENT_TYPE_MAP[major]
            return FileAttachment

        assets = await self.get_assets()
        items: list[Attachment] = []
        for asset in assets:
            cls = _pick_cls(asset.content_type)
            items.append(
                cls(
                    name=asset.name,
                    url=asset.browser_download_url,
                    size=asset.size,
                    content_type=asset.content_type or None,
                    extra={
                        "id": asset.id,
                        "download_count": asset.download_count,
                    },
                )
            )
        return AttachmentList(items)



"""GitHub 平台发送者类型"""

from __future__ import annotations

from typing import Optional

from ncatbot.types.common.sender import BaseSender

from .enums import GitHubUserType

__all__ = [
    "GitHubSender",
]


class GitHubSender(BaseSender):
    """GitHub 用户信息"""

    login: str = ""
    avatar_url: Optional[str] = None
    html_url: Optional[str] = None
    user_type: GitHubUserType = GitHubUserType.USER



"""
Predicate 语法糖系统

为 ``wait_event`` 的 ``predicate`` 参数提供可组合的声明式 DSL，
替代冗长的 lambda 表达式。

运算符
------
- ``p1 * p2`` / ``p1 & p2`` — AND（全部满足）
- ``p1 + p2`` / ``p1 | p2`` — OR（任一满足）
- ``~p`` — NOT（取反）

快速开始::

    from ncatbot.core import from_event, msg_equals

    # 等待同 session 的回复
    evt = await self.wait_event(predicate=from_event(event), timeout=30)

    # 同 session + 精确匹配
    evt = await self.wait_event(
        predicate=from_event(event) * msg_equals("确认"),
        timeout=15,
    )
"""

from __future__ import annotations

import re
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Callable, Sequence, Union

if TYPE_CHECKING:
    from .event import Event

__all__ = [
    # 基类 & 组合
    "P",
    "AndP",
    "OrP",
    "NotP",
    # 工厂函数
    "same_user",
    "same_group",
    "is_private",
    "is_group",
    "is_message",
    "has_keyword",
    "msg_equals",
    "msg_in",
    "msg_matches",
    "event_type",
    "from_event",
]


# ---------------------------------------------------------------------------
# 基类
# ---------------------------------------------------------------------------


class P(ABC):
    """Predicate 抽象基类，可直接作为 ``Callable[[Event], bool]`` 传给 ``wait_event``。"""

    @abstractmethod
    def __call__(self, event: "Event") -> bool: ...

    # ---- 组合运算符 ----

    def __mul__(self, other: Union["P", Callable[["Event"], bool]]) -> "AndP":
        """``p1 * p2`` → AND"""
        return AndP(self, _ensure_p(other))

    def __and__(self, other: Union["P", Callable[["Event"], bool]]) -> "AndP":
        """``p1 & p2`` → AND"""
        return AndP(self, _ensure_p(other))

    def __add__(self, other: Union["P", Callable[["Event"], bool]]) -> "OrP":
        """``p1 + p2`` → OR"""
        return OrP(self, _ensure_p(other))

    def __or__(self, other: Union["P", Callable[["Event"], bool]]) -> "OrP":
        """``p1 | p2`` → OR"""
        return OrP(self, _ensure_p(other))

    def __invert__(self) -> "NotP":
        """``~p`` → NOT"""
        return NotP(self)

    # ---- 工具方法 ----

    @classmethod
    def of(cls, fn: Callable[["Event"], bool]) -> "P":
        """将普通 callable / lambda 升级为 :class:`P` 实例，使其支持运算符组合。"""
        if isinstance(fn, P):
            return fn
        return _LambdaP(fn)


# ---------------------------------------------------------------------------
# 组合类
# ---------------------------------------------------------------------------


class AndP(P):
    """全部子谓词均满足。"""

    __slots__ = ("_children",)

    def __init__(self, *children: P) -> None:
        # 展平嵌套 AndP，减少调用层级
        flat: list[P] = []
        for c in children:
            if isinstance(c, AndP):
                flat.extend(c._children)
            else:
                flat.append(c)
        self._children: tuple[P, ...] = tuple(flat)

    def __call__(self, event: "Event") -> bool:
        return all(c(event) for c in self._children)

    def __repr__(self) -> str:
        inner = ", ".join(repr(c) for c in self._children)
        return f"AndP({inner})"


class OrP(P):
    """任一子谓词满足。"""

    __slots__ = ("_children",)

    def __init__(self, *children: P) -> None:
        flat: list[P] = []
        for c in children:
            if isinstance(c, OrP):
                flat.extend(c._children)
            else:
                flat.append(c)
        self._children: tuple[P, ...] = tuple(flat)

    def __call__(self, event: "Event") -> bool:
        return any(c(event) for c in self._children)

    def __repr__(self) -> str:
        inner = ", ".join(repr(c) for c in self._children)
        return f"OrP({inner})"


class NotP(P):
    """取反。"""

    __slots__ = ("_inner",)

    def __init__(self, inner: P) -> None:
        self._inner = inner

    def __call__(self, event: "Event") -> bool:
        return not self._inner(event)

    def __repr__(self) -> str:
        return f"NotP({self._inner!r})"


class _LambdaP(P):
    """包装普通 callable 为 :class:`P`。"""

    __slots__ = ("_fn",)

    def __init__(self, fn: Callable[["Event"], bool]) -> None:
        self._fn = fn

    def __call__(self, event: "Event") -> bool:
        return self._fn(event)

    def __repr__(self) -> str:
        return f"P.of({self._fn!r})"


def _ensure_p(obj: Union[P, Callable[["Event"], bool]]) -> P:
    """确保对象为 P 实例。"""
    if isinstance(obj, P):
        return obj
    return _LambdaP(obj)


# ---------------------------------------------------------------------------
# 安全属性访问辅助
# ---------------------------------------------------------------------------


def _safe_raw_message(event: "Event") -> str | None:
    """安全获取 raw_message，不存在返回 None。"""
    data = event.data
    if hasattr(data, "raw_message"):
        return data.raw_message  # type: ignore[union-attr]
    return None


# ---------------------------------------------------------------------------
# 预置工厂函数
# ---------------------------------------------------------------------------


class _SameUser(P):
    __slots__ = ("_uid",)

    def __init__(self, uid: str) -> None:
        self._uid = uid

    def __call__(self, event: "Event") -> bool:
        return hasattr(event.data, "user_id") and str(event.data.user_id) == self._uid  # type: ignore[union-attr]

    def __repr__(self) -> str:
        return f"same_user({self._uid!r})"


def same_user(user_id: Union[str, int]) -> P:
    """匹配 ``event.data.user_id``。"""
    return _SameUser(str(user_id))


class _SameGroup(P):
    __slots__ = ("_gid",)

    def __init__(self, gid: str) -> None:
        self._gid = gid

    def __call__(self, event: "Event") -> bool:
        return hasattr(event.data, "group_id") and str(event.data.group_id) == self._gid  # type: ignore[union-attr]

    def __repr__(self) -> str:
        return f"same_group({self._gid!r})"


def same_group(group_id: Union[str, int]) -> P:
    """匹配 ``event.data.group_id``。"""
    return _SameGroup(str(group_id))


class _EventType(P):
    __slots__ = ("_prefix",)

    def __init__(self, prefix: str) -> None:
        self._prefix = prefix

    def __call__(self, event: "Event") -> bool:
        t = event.type
        return t == self._prefix or t.startswith(self._prefix + ".")

    def __repr__(self) -> str:
        return f"event_type({self._prefix!r})"


def event_type(prefix: str) -> P:
    """匹配 ``event.type`` 前缀（含精确匹配），如 ``"message"``、``"message.group"``。"""
    return _EventType(prefix)


# 单例实例，避免重复创建

_IS_PRIVATE = _EventType("message.private")
_IS_GROUP = _EventType("message.group")
_IS_MESSAGE = _EventType("message")


def is_private() -> P:
    """事件类型为私聊消息。"""
    return _IS_PRIVATE


def is_group() -> P:
    """事件类型为群消息。"""
    return _IS_GROUP


def is_message() -> P:
    """事件类型为消息（群或私聊）。"""
    return _IS_MESSAGE


class _HasKeyword(P):
    __slots__ = ("_words",)

    def __init__(self, words: Sequence[str]) -> None:
        self._words = tuple(words)

    def __call__(self, event: "Event") -> bool:
        text = _safe_raw_message(event)
        if text is None:
            return False
        return any(w in text for w in self._words)

    def __repr__(self) -> str:
        inner = ", ".join(repr(w) for w in self._words)
        return f"has_keyword({inner})"


def has_keyword(*words: str) -> P:
    """``raw_message`` 包含任一关键词。"""
    return _HasKeyword(words)


class _MsgEquals(P):
    __slots__ = ("_text",)

    def __init__(self, text: str) -> None:
        self._text = text

    def __call__(self, event: "Event") -> bool:
        raw = _safe_raw_message(event)
        if raw is None:
            return False
        return raw.strip() == self._text

    def __repr__(self) -> str:
        return f"msg_equals({self._text!r})"


def msg_equals(text: str) -> P:
    """``raw_message.strip()`` 完全匹配。"""
    return _MsgEquals(text)


class _MsgIn(P):
    __slots__ = ("_options",)

    def __init__(self, options: Sequence[str]) -> None:
        self._options = frozenset(options)

    def __call__(self, event: "Event") -> bool:
        raw = _safe_raw_message(event)
        if raw is None:
            return False
        return raw.strip() in self._options

    def __repr__(self) -> str:
        inner = ", ".join(repr(o) for o in sorted(self._options))
        return f"msg_in({inner})"


def msg_in(*options: str) -> P:
    """``raw_message.strip()`` 匹配选项之一，等价于多个 ``msg_equals`` 的 OR。"""
    return _MsgIn(options)


class _MsgMatches(P):
    __slots__ = ("_pattern", "_compiled")

    def __init__(self, pattern: str) -> None:
        self._pattern = pattern
        self._compiled = re.compile(pattern)

    def __call__(self, event: "Event") -> bool:
        raw = _safe_raw_message(event)
        if raw is None:
            return False
        return self._compiled.search(raw) is not None

    def __repr__(self) -> str:
        return f"msg_matches({self._pattern!r})"


def msg_matches(pattern: str) -> P:
    """``raw_message`` 正则匹配（search 模式）。"""
    return _MsgMatches(pattern)


# ---------------------------------------------------------------------------
# 核心语法糖：from_event
# ---------------------------------------------------------------------------


def from_event(event: object) -> P:
    """从触发事件自动推导「同 session」谓词。

    - **群消息事件** → ``same_user(uid) * same_group(gid) * is_group()``
    - **私聊消息事件** → ``same_user(uid) * is_private()``
    - **其他带 user_id 的事件** → ``same_user(uid) * is_message()``

    同时支持高层事件对象（``BaseEvent``，有 ``.user_id``）
    和底层 ``Event``（有 ``.data.user_id``）。
    """
    # 统一提取 user_id 和 group_id
    uid: str | None = None
    gid: str | None = None
    is_group_msg = False
    is_private_msg = False

    # 高层事件 (BaseEvent 及其子类): 有 .user_id 属性
    if hasattr(event, "user_id") and not hasattr(event, "type"):
        uid = str(event.user_id)  # type: ignore[union-attr]
        if hasattr(event, "group_id"):
            gid = str(event.group_id)  # type: ignore[union-attr]
            is_group_msg = True
        elif hasattr(event, "message_type"):
            from ncatbot.types.qq import MessageType

            is_private_msg = event.message_type is MessageType.PRIVATE  # type: ignore[union-attr]

    # 底层 Event (dispatcher Event): 有 .type 和 .data
    elif hasattr(event, "type") and hasattr(event, "data"):
        data = event.data  # type: ignore[union-attr]
        event_type_str: str = event.type  # type: ignore[union-attr]

        if hasattr(data, "user_id"):
            uid = str(data.user_id)
        if hasattr(data, "group_id"):
            gid = str(data.group_id)

        is_group_msg = event_type_str == "message.group"
        is_private_msg = event_type_str == "message.private"

    if uid is None:
        raise ValueError(
            f"from_event: 无法从 {type(event).__name__} 中提取 user_id，"
            "请手动组合 predicate"
        )

    if is_group_msg and gid is not None:
        return same_user(uid) * same_group(gid) * is_group()
    if is_private_msg:
        return same_user(uid) * is_private()
    # fallback: 只匹配用户 + 消息事件
    return same_user(uid) * is_message()



"""Comment API Mixin"""

from __future__ import annotations

from typing import Any, List

from ncatbot.types.github.models import GitHubCommentInfo


class CommentAPIMixin:
    """评论操作"""

    async def _request(self, method: str, path: str, **kwargs: Any) -> Any: ...

    async def create_issue_comment(
        self, repo: str, issue_number: int, body: str
    ) -> GitHubCommentInfo:
        data = await self._request(
            "POST",
            f"/repos/{repo}/issues/{issue_number}/comments",
            json={"body": body},
        )
        return GitHubCommentInfo.model_validate(data)

    async def update_comment(
        self, repo: str, comment_id: int, body: str
    ) -> GitHubCommentInfo:
        data = await self._request(
            "PATCH",
            f"/repos/{repo}/issues/comments/{comment_id}",
            json={"body": body},
        )
        return GitHubCommentInfo.model_validate(data)

    async def delete_comment(self, repo: str, comment_id: int) -> None:
        await self._request("DELETE", f"/repos/{repo}/issues/comments/{comment_id}")

    async def list_issue_comments(
        self, repo: str, issue_number: int, page: int = 1, per_page: int = 30
    ) -> List[GitHubCommentInfo]:
        data = await self._request(
            "GET",
            f"/repos/{repo}/issues/{issue_number}/comments",
            params={"page": page, "per_page": per_page},
        )
        return [GitHubCommentInfo.model_validate(item) for item in data]



"""
事件消费混入类

代理 AsyncEventDispatcher 的 events() / wait_event() 接口，
通过 _mixin_unload 钩子在卸载时自动关闭所有活跃的 EventStream。
"""

from typing import Callable, List, Optional, Union, TYPE_CHECKING

from ncatbot.utils import get_log

if TYPE_CHECKING:
    from ncatbot.core import AsyncEventDispatcher, Event, EventStream
    from ncatbot.types.qq import EventType

LOG = get_log("EventMixin")


class EventMixin:
    """
    事件消费混入类

    - ``_mixin_unload``: 关闭所有活跃的 EventStream

    使用示例::

        async with self.events("message") as stream:
            async for event in stream:
                ...
    """

    _dispatcher: "AsyncEventDispatcher"

    def events(
        self,
        event_type: Optional[Union[str, "EventType"]] = None,
    ) -> "EventStream":
        """创建事件流，可选按类型过滤。

        返回的 EventStream 支持 async with / async for。
        插件卸载时框架会自动关闭所有未手动关闭的流。

        Args:
            event_type: 事件类型过滤（前缀匹配），如 "message"、"notice" 等

        Returns:
            EventStream 异步迭代器
        """
        stream = self._dispatcher.events(event_type)
        if not hasattr(self, "_active_streams"):
            self._active_streams: List["EventStream"] = []
        self._active_streams.append(stream)
        return stream

    async def wait_event(
        self,
        predicate: Optional[Callable[["Event"], bool]] = None,
        timeout: Optional[float] = None,
    ) -> "Event":
        """等待下一个满足条件的事件。

        Args:
            predicate: 过滤函数，None 表示接受任意事件
            timeout: 超时秒数，None 表示无限等待

        Returns:
            匹配的 Event

        Raises:
            asyncio.TimeoutError: 超时未匹配到事件
        """
        return await self._dispatcher.wait_event(predicate, timeout)

    # ------------------------------------------------------------------
    # Mixin 钩子
    # ------------------------------------------------------------------

    async def _mixin_unload(self) -> None:
        """关闭所有活跃的 EventStream。"""
        if not hasattr(self, "_active_streams"):
            return
        for stream in self._active_streams:
            try:
                await stream.aclose()
            except Exception:
                pass
        self._active_streams.clear()



"""MessageArray — 消息段容器 + 查询 + 链式构造"""

from __future__ import annotations

from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    Iterable,
    List,
    Optional,
    Type,
    TypeVar,
    Union,
    overload,
)

from pydantic import GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema

from .base import MessageSegment, parse_segment
from .text import At, PlainText, Reply
from .media import DownloadableSegment, Image, Video

if TYPE_CHECKING:
    from ncatbot.types.common.attachment_list import AttachmentList

__all__ = [
    "MessageArray",
]

T = TypeVar("T", bound=MessageSegment)


def _parse_any(data: Any) -> List[MessageSegment]:
    """将各种输入格式统一解析为 MessageSegment 列表"""
    if isinstance(data, MessageSegment):
        return [data]
    if isinstance(data, dict):
        return [parse_segment(data)]
    if isinstance(data, str):
        return [PlainText(text=data)]
    if isinstance(data, Iterable):
        result: List[MessageSegment] = []
        for item in data:
            result.extend(_parse_any(item))
        return result
    return []


class MessageArray:
    """消息段数组 — 容器 + 查询 + 链式构造"""

    __slots__ = ("_segments",)

    def __init__(self, segments: Optional[List[MessageSegment]] = None):
        self._segments: List[MessageSegment] = list(segments or [])

    # ---- Pydantic 自定义类型支持 ----

    @classmethod
    def __get_pydantic_core_schema__(
        cls, source_type: Any, handler: GetCoreSchemaHandler
    ) -> CoreSchema:
        return core_schema.no_info_plain_validator_function(
            cls._pydantic_validate,
            serialization=core_schema.plain_serializer_function_ser_schema(
                cls._pydantic_serialize, info_arg=False
            ),
        )

    @classmethod
    def _pydantic_validate(cls, v: Any) -> MessageArray:
        if isinstance(v, MessageArray):
            return v
        if isinstance(v, list):
            return cls.from_list(v)
        if isinstance(v, str):
            return cls.from_any(v)
        raise ValueError(f"Cannot convert {type(v)} to MessageArray")

    def _pydantic_serialize(self) -> List[Dict[str, Any]]:
        return self.to_list()

    # ---- 反序列化 ----

    @classmethod
    def from_list(cls, data: List[Dict[str, Any]]) -> MessageArray:
        return cls([parse_segment(seg) for seg in data])

    @classmethod
    def from_any(cls, data: Any) -> MessageArray:
        return cls(_parse_any(data))

    # ---- 序列化 ----

    def to_list(self) -> List[Dict[str, Any]]:
        return [seg.to_dict() for seg in self._segments]

    # ---- 查询 ----

    @overload
    def filter(self, cls: None = None) -> List[MessageSegment]: ...

    @overload
    def filter(self, cls: Type[T]) -> List[T]: ...

    def filter(
        self, cls: Union[Type[T], None] = None
    ) -> Union[List[MessageSegment], List[T]]:
        if cls is None:
            return list(self._segments)
        return [s for s in self._segments if isinstance(s, cls)]

    def filter_text(self) -> List[PlainText]:
        return self.filter(PlainText)

    def filter_at(self) -> List[At]:
        return self.filter(At)

    def filter_image(self) -> List[Image]:
        return self.filter(Image)

    def filter_video(self) -> List[Video]:
        return self.filter(Video)

    def get_attachments(self) -> "AttachmentList":
        """提取所有可下载段为 AttachmentList"""
        from ncatbot.types.common.attachment_list import AttachmentList

        return AttachmentList(
            seg.to_attachment()
            for seg in self._segments
            if isinstance(seg, DownloadableSegment) and (seg.url or seg.file)
        )

    @property
    def text(self) -> str:
        """拼接所有纯文本段"""
        return "".join(s.text for s in self.filter(PlainText))

    def is_at(self, user_id: Union[str, int], all_except: bool = False) -> bool:
        uid = str(user_id)
        all_at = False
        for at in self.filter(At):
            if at.user_id == uid:
                return True
            if at.user_id == "all":
                all_at = True
        return not all_except and all_at

    # ---- 链式构造 ----

    def add_text(self, text: str) -> MessageArray:
        self._segments.extend(_parse_any(text))
        return self

    def add_image(self, image: Union[str, Image]) -> MessageArray:
        if isinstance(image, Image):
            self._segments.append(image)
        elif isinstance(image, str):
            self._segments.append(Image(file=image))
        else:
            raise TypeError(f"image must be str or Image, got {type(image)}")
        return self

    def add_video(self, video: Union[str, Video]) -> MessageArray:
        if isinstance(video, Video):
            self._segments.append(video)
        elif isinstance(video, str):
            self._segments.append(Video(file=video))
        else:
            raise TypeError(f"video must be str or Video, got {type(video)}")
        return self

    def add_at(self, user_id: Union[str, int]) -> MessageArray:
        self._segments.append(At(user_id=str(user_id)))
        return self

    def add_at_all(self) -> MessageArray:
        self._segments.append(At(user_id="all"))
        return self

    def add_reply(self, message_id: Union[str, int]) -> MessageArray:
        self._segments.append(Reply(id=str(message_id)))
        return self

    def add_segment(self, segment: MessageSegment) -> MessageArray:
        self._segments.append(segment)
        return self

    def add_segments(self, data: Any) -> MessageArray:
        self._segments.extend(_parse_any(data))
        return self

    # ---- 容器协议 ----

    def __iter__(self):
        return iter(self._segments)

    def __len__(self):
        return len(self._segments)

    def __add__(self, other: Any) -> MessageArray:
        return MessageArray(self._segments + _parse_any(other))

    def __radd__(self, other: Any) -> MessageArray:
        return MessageArray(_parse_any(other) + self._segments)

    def __repr__(self) -> str:
        return f"MessageArray({self._segments!r})"



"""QQ 专用转发消息段"""

from __future__ import annotations

from typing import Any, ClassVar, Dict, List, Optional

from pydantic import BaseModel, field_validator

from ncatbot.types.common.segment.base import MessageSegment, parse_segment

__all__ = [
    "ForwardNode",
    "Forward",
]


class ForwardNode(BaseModel):
    """转发消息的单个节点"""

    user_id: str
    nickname: str
    content: List[MessageSegment]

    @field_validator("user_id", mode="before")
    @classmethod
    def _coerce_uid(cls, v: object) -> str:
        return str(v) if v is not None else ""

    @field_validator("content", mode="before")
    @classmethod
    def _parse_content(cls, v: object) -> List[MessageSegment]:
        if isinstance(v, list):
            result: List[MessageSegment] = []
            for item in v:
                if isinstance(item, dict):
                    result.append(parse_segment(item))
                elif isinstance(item, MessageSegment):
                    result.append(item)
            return result
        return []

    def to_node_dict(self) -> Dict[str, Any]:
        content_list: List[Dict[str, Any]] = []
        for seg in self.content:
            if isinstance(seg, Forward) and seg.content:
                for inner_node in seg.content:
                    content_list.append(inner_node.to_node_dict())
            else:
                content_list.append(seg.to_dict())
        return {
            "type": "node",
            "data": {
                "name": self.nickname,
                "uin": self.user_id,
                "content": content_list,
            },
        }


class Forward(MessageSegment):
    _type: ClassVar[str] = "forward"
    id: Optional[str] = None
    content: Optional[List[ForwardNode]] = None

    def to_dict(self) -> Dict[str, Any]:
        if self.content:
            return {
                "type": "forward",
                "data": {"content": [node.to_node_dict() for node in self.content]},
            }
        if self.id:
            return {"type": "forward", "data": {"id": self.id}}
        return {"type": "forward", "data": {}}

    def to_forward_dict(self) -> Dict[str, Any]:
        if not self.content:
            return {"messages": []}
        return {"messages": [node.to_node_dict() for node in self.content]}

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> Forward:
        seg_data = dict(data.get("data", {}))
        content = seg_data.get("content")
        if isinstance(content, list):
            valid: List[Dict[str, Any]] = []
            for item in content:
                if not isinstance(item, dict):
                    continue
                if "message" in item:
                    node_data = {
                        "user_id": item.get("user_id", ""),
                        "nickname": item.get("sender", {}).get("nickname", ""),
                        "content": item.get("message", []),
                    }
                    valid.append(node_data)
                else:
                    valid.append(item)
            seg_data["content"] = valid or None
        return cls.model_validate(seg_data)



"""用户相关 API 响应类型"""

from __future__ import annotations

from typing import Optional

from ._base import NapCatModel


class LoginInfo(NapCatModel):
    """登录账号信息

    对应: ``get_login_info``
    """

    user_id: str = ""
    nickname: Optional[str] = None


class StrangerInfo(NapCatModel):
    """陌生人信息

    对应: ``get_stranger_info``
    """

    user_id: str = ""
    nickname: Optional[str] = None
    sex: Optional[str] = None
    age: Optional[int] = None


class FriendInfo(NapCatModel):
    """好友信息

    对应: ``get_friend_list`` 中的每一项
    """

    user_id: str = ""
    nickname: Optional[str] = None
    remark: Optional[str] = None



"""Bilibili 平台专用事件实体"""

from .live import (
    BiliLiveEvent,
    DanmuMsgEvent,
    SuperChatEvent,
    GiftEvent,
    GuardBuyEvent,
    InteractEvent,
    LikeEvent,
    LiveNoticeEvent,
)
from .session import BiliPrivateMessageEvent, BiliPrivateMessageWithdrawEvent
from .comment import BiliCommentEvent
from .dynamic import BiliDynamicEvent
from .factory import create_bili_entity

# 自动注册 Bilibili 平台工厂和 secondary keys 到通用工厂
from ncatbot.event.common.factory import (
    register_platform_factory as _register,
    register_platform_secondary_keys as _register_keys,
)

_register("bilibili", create_bili_entity)
_register_keys(
    "bilibili",
    {
        "live": "live_event_type",
        "comment": "comment_event_type",
        "dynamic": "dynamic_event_type",
    },
)
del _register, _register_keys

__all__ = [
    # live
    "BiliLiveEvent",
    "DanmuMsgEvent",
    "SuperChatEvent",
    "GiftEvent",
    "GuardBuyEvent",
    "InteractEvent",
    "LikeEvent",
    "LiveNoticeEvent",
    # session
    "BiliPrivateMessageEvent",
    "BiliPrivateMessageWithdrawEvent",
    # comment
    "BiliCommentEvent",
    # dynamic
    "BiliDynamicEvent",
    # factory
    "create_bili_entity",
]



"""跨平台通用附件模型"""

from __future__ import annotations

from enum import Enum
from pathlib import Path
from typing import Any, Dict, Literal, Optional, Union

from pydantic import BaseModel, Field
from ncatbot.utils.network import async_download_to_bytes, async_download_to_file
from ncatbot.utils.config import get_config_manager

from .segment.base import MessageSegment
from .segment.media import File, Image, Video, Record

__all__ = [
    "AttachmentKind",
    "Attachment",
    "ImageAttachment",
    "VideoAttachment",
    "AudioAttachment",
    "FileAttachment",
]


class AttachmentKind(str, Enum):
    """附件类型标识"""

    IMAGE = "image"
    VIDEO = "video"
    AUDIO = "audio"
    FILE = "file"
    OTHER = "other"


class Attachment(BaseModel):
    """可下载附件的跨平台描述

    各平台事件实体通过 ``HasAttachments`` mixin 将平台原生附件
    转换为 ``Attachment``，插件侧可统一处理。
    """

    name: str
    url: str
    size: Optional[int] = None
    content_type: Optional[str] = None
    kind: AttachmentKind = AttachmentKind.OTHER
    extra: Dict[str, Any] = Field(default_factory=dict)

    async def download(
        self, dest: Union[str, Path], *, proxy: Optional[str] = None
    ) -> Path:
        """下载附件到本地目录

        Parameters
        ----------
        dest : str | Path
            目标**目录**。文件将保存为 ``dest / self.name``。
        proxy : str, optional
            HTTP/SOCKS5 代理地址。省略时自动读取主配置 ``http_proxy``；
            配置也为空则直连。

        Returns
        -------
        Path
            实际写入的文件路径。
        """
        if proxy is None:
            try:
                proxy = get_config_manager().config.http_proxy or None
            except Exception:
                proxy = None

        return await async_download_to_file(
            self.url, dest, filename=self.name, proxy=proxy
        )

    async def as_bytes(self, *, proxy: Optional[str] = None) -> bytes:
        """下载到内存，返回原始字节

        Parameters
        ----------
        proxy : str, optional
            HTTP/SOCKS5 代理地址。省略时自动读取主配置 ``http_proxy``。
        """
        if proxy is None:
            try:
                proxy = get_config_manager().config.http_proxy or None
            except Exception:
                proxy = None

        return await async_download_to_bytes(self.url, proxy=proxy)

    def to_segment(self) -> MessageSegment:
        """转为消息段（直接使用 URL，同平台内转发最快）"""
        return File(file=self.url, file_name=self.name, file_size=self.size)

    async def to_local_segment(
        self, cache_dir: Union[str, Path] = ".cache/attachments"
    ) -> MessageSegment:
        """先下载到本地，再转为消息段（跨平台安全）"""
        local = await self.download(cache_dir)
        return File(file=str(local), file_name=self.name, file_size=self.size)


class ImageAttachment(Attachment):
    """图片附件"""

    kind: Literal[AttachmentKind.IMAGE] = AttachmentKind.IMAGE
    width: Optional[int] = None
    height: Optional[int] = None

    def to_segment(self) -> MessageSegment:
        return Image(file=self.url, file_name=self.name, file_size=self.size)

    async def to_local_segment(
        self, cache_dir: Union[str, Path] = ".cache/attachments"
    ) -> MessageSegment:
        local = await self.download(cache_dir)
        return Image(file=str(local), file_name=self.name, file_size=self.size)


class VideoAttachment(Attachment):
    """视频附件"""

    kind: Literal[AttachmentKind.VIDEO] = AttachmentKind.VIDEO
    duration: Optional[int] = None

    def to_segment(self) -> MessageSegment:
        return Video(file=self.url, file_name=self.name, file_size=self.size)

    async def to_local_segment(
        self, cache_dir: Union[str, Path] = ".cache/attachments"
    ) -> MessageSegment:
        local = await self.download(cache_dir)
        return Video(file=str(local), file_name=self.name, file_size=self.size)


class AudioAttachment(Attachment):
    """音频附件"""

    kind: Literal[AttachmentKind.AUDIO] = AttachmentKind.AUDIO
    duration: Optional[int] = None

    def to_segment(self) -> MessageSegment:
        return Record(file=self.url, file_name=self.name, file_size=self.size)

    async def to_local_segment(
        self, cache_dir: Union[str, Path] = ".cache/attachments"
    ) -> MessageSegment:
        local = await self.download(cache_dir)
        return Record(file=str(local), file_name=self.name, file_size=self.size)


class FileAttachment(Attachment):
    """文件附件"""

    kind: Literal[AttachmentKind.FILE] = AttachmentKind.FILE

    def to_segment(self) -> MessageSegment:
        return File(file=self.url, file_name=self.name, file_size=self.size)

    async def to_local_segment(
        self, cache_dir: Union[str, Path] = ".cache/attachments"
    ) -> MessageSegment:
        local = await self.download(cache_dir)
        return File(file=str(local), file_name=self.name, file_size=self.size)



"""QQ 平台专用枚举"""

from enum import Enum

__all__ = [
    "PostType",
    "MessageType",
    "NoticeType",
    "NotifySubType",
    "RequestType",
    "MetaEventType",
    "EventType",
]


class PostType(str, Enum):
    MESSAGE = "message"
    MESSAGE_SENT = "message_sent"
    NOTICE = "notice"
    REQUEST = "request"
    META_EVENT = "meta_event"


class MessageType(str, Enum):
    PRIVATE = "private"
    GROUP = "group"


class NoticeType(str, Enum):
    GROUP_UPLOAD = "group_upload"
    GROUP_ADMIN = "group_admin"
    GROUP_DECREASE = "group_decrease"
    GROUP_INCREASE = "group_increase"
    GROUP_BAN = "group_ban"
    FRIEND_ADD = "friend_add"
    GROUP_RECALL = "group_recall"
    FRIEND_RECALL = "friend_recall"
    NOTIFY = "notify"


class NotifySubType(str, Enum):
    POKE = "poke"
    LUCKY_KING = "lucky_king"
    HONOR = "honor"


class RequestType(str, Enum):
    FRIEND = "friend"
    GROUP = "group"


class MetaEventType(str, Enum):
    LIFECYCLE = "lifecycle"
    HEARTBEAT = "heartbeat"
    HEARTBEAT_TIMEOUT = "heartbeat_timeout"


class EventType(str, Enum):
    MESSAGE = "message_event"
    MESSAGE_SENT = "message_sent_event"
    NOTICE = "notice_event"
    REQUEST = "request_event"
    META = "meta_event"



"""
注册系统 (Registry)

Hook 驱动 + HandlerDispatcher 分发 + Registrar 装饰器 + ContextVar 隔离。

核心组件:
- Hook: 绑定在 handler 上的拦截器 (BEFORE_CALL / AFTER_CALL / ON_ERROR)
- HandlerDispatcher: 纯机械分发器，通过 listener 接收 Event → 收集 handler → 执行 Hook 链
- Registrar: ContextVar 驱动的装饰器集合，按插件隔离收集 handler
- ContextVar: PluginLoader 在 exec_module 前后设置/重置，装饰器内读取
- PlatformRegistrar: 平台子注册器，提供平台专属便捷装饰器

事件类型格式统一使用 BaseEventData.resolve_type() 产出的格式:
  QQ: "message.group"、"notice.group_increase"、"request.friend" 等。
  Bilibili: "live.danmu_msg"、"live.send_gift"、"comment.new_reply" 等。
  GitHub: "issue.opened"、"pull_request.closed"、"push" 等。
"""

# ContextVar
from .context import set_current_plugin, get_current_plugin

# Hook 系统
from .hook import Hook, HookStage, HookAction, HookContext, add_hooks, get_hooks

# 内置 Hook
from .builtin_hooks import (
    MessageTypeFilter,
    PostTypeFilter,
    SubTypeFilter,
    SelfFilter,
    StartsWithHook,
    KeywordHook,
    RegexHook,
    NoticeTypeFilter,
    RequestTypeFilter,
    PlatformFilter,
    group_only,
    private_only,
    non_self,
    startswith,
    keyword,
    regex,
)

# 命令 Hook
from .command_hook import CommandHook

# 命令组 Hook
from .command_group_hook import CommandGroup, CommandGroupHook

# Dispatcher
from .dispatcher import HandlerDispatcher, HandlerEntry

# Registrar
from .registrar import Registrar, registrar, flush_pending, clear_pending

# 平台子注册器
from .platform import PlatformRegistrar, QQRegistrar, BilibiliRegistrar, GitHubRegistrar

__all__ = [
    # ContextVar
    "set_current_plugin",
    "get_current_plugin",
    # Hook
    "Hook",
    "HookStage",
    "HookAction",
    "HookContext",
    "add_hooks",
    "get_hooks",
    # 内置 Hook (低级过滤)
    "MessageTypeFilter",
    "PostTypeFilter",
    "SubTypeFilter",
    "SelfFilter",
    "StartsWithHook",
    "KeywordHook",
    "RegexHook",
    "NoticeTypeFilter",
    "RequestTypeFilter",
    "PlatformFilter",
    "group_only",
    "private_only",
    "non_self",
    "startswith",
    "keyword",
    "regex",
    # 命令 Hook (高级匹配 + 参数绑定)
    "CommandHook",
    "CommandGroup",
    "CommandGroupHook",
    # Dispatcher
    "HandlerDispatcher",
    "HandlerEntry",
    # Registrar
    "Registrar",
    "registrar",
    "flush_pending",
    "clear_pending",
    # 平台子注册器
    "PlatformRegistrar",
    "QQRegistrar",
    "BilibiliRegistrar",
    "GitHubRegistrar",
]



"""数据源抽象基类"""

from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Awaitable, Callable


class BaseSource(ABC):
    """单一数据源抽象 — 每个实例对应一路事件流。

    回调签名: ``async callback(source_type: str, raw_data: dict) -> None``
    """

    source_type: str  # "live" / "session" / "comment"
    source_id: str  # room_id / "session" / resource_id

    def __init__(
        self,
        callback: Callable[[str, dict], Awaitable[None]],
    ) -> None:
        self._callback = callback
        self._running = False

    @abstractmethod
    async def start(self) -> None:
        """启动数据源（非阻塞，内部自行管理 task）"""

    @abstractmethod
    async def stop(self) -> None:
        """停止数据源，释放资源"""

    @property
    def running(self) -> bool:
        return self._running

    def __repr__(self) -> str:
        return f"{type(self).__name__}({self.source_type}:{self.source_id}, running={self._running})"



"""
CommandHook — 命令匹配 + 注解式参数绑定

高级 BEFORE_CALL Hook:
1. 预处理消息段：将首个 PlainText 移到 index 0
2. 使用 shlex 分词后的首 token 统一前缀匹配命令名
3. 构建 binding stream：命令后剩余文本 + 后续段
4. 从 binding stream 中按类型注解逐项绑定 (str/int/float/At)
5. 支持引号（双引号/单引号包裹部分作为单个 token）
6. 不匹配的段被跳过（永久消耗）
7. 必选参数缺失时 WARNING + 回复 usage + SKIP
8. 写入 ctx.kwargs → dispatcher._execute(**ctx.kwargs) 自动注入
"""

from typing import Dict, Optional

from ncatbot.utils import get_log

from .hook import Hook, HookAction, HookContext, HookStage
from ._command_binding import (
    _ParamSpec,
    preprocess_segments,
    build_binding_stream,
    bind_params,
    get_param_spec,
    reply_usage,
    match_command_prefix,
)

LOG = get_log("CommandHook")


class CommandHook(Hook):
    """命令匹配 + 注解式参数绑定

    匹配规则:
    - 统一前缀匹配 — 对消息预处理后首段 PlainText 分词,
      首 token 匹配命令名即触发（无论 handler 有无额外参数）

    参数绑定规则:
    - 消息段预处理: 首个 PlainText 移到最前, 解决 Reply 开头的消息
    - 引号支持: ``"hello world"`` 作为单个 token
    - At 注解 → 从 binding stream 中匹配 kind="at" 的项
    - int/float → 从 binding stream 中匹配 kind="token" 并转换
    - str → 单 token 或剩余全部文本 (最后一个 str 参数)
    - 不匹配的段被跳过（永久消耗, 不再参与后续参数匹配）
    - 有默认值 → 可选; 必选参数缺失 → WARNING + 回复 usage + SKIP
    """

    stage = HookStage.BEFORE_CALL

    def __init__(
        self,
        *names: str,
        ignore_case: bool = False,
        priority: int = 95,
    ):
        if not names:
            raise ValueError("CommandHook 至少需要一个命令名")
        self.names = names
        self.ignore_case = ignore_case
        self.priority = priority
        self._sig_cache: Dict[int, Optional[_ParamSpec]] = {}

    async def execute(self, ctx: HookContext) -> HookAction:
        # 获取消息
        message = getattr(ctx.event.data, "message", None)
        if message is None:
            return HookAction.SKIP

        # 1) 预处理: 首个 PlainText 移到最前
        segments = preprocess_segments(message)
        if not segments:
            return HookAction.SKIP

        # 2) 从首段 PlainText 取文本
        from ncatbot.types.common.segment.text import PlainText

        first_seg = segments[0]
        if not isinstance(first_seg, PlainText):
            return HookAction.SKIP

        text = first_seg.text.strip()
        if not text:
            return HookAction.SKIP

        # 3) 统一前缀匹配: tokenize 后首 token 匹配命令名
        matched_name = match_command_prefix(text, self.names, self.ignore_case)
        if matched_name is None:
            return HookAction.SKIP

        # 4) 解析 handler 参数规格 (缓存)
        func = ctx.handler_entry.func
        spec = self._get_param_spec(func)

        if spec is None or not spec.params:
            # 无额外参数 → 命令已匹配, 直接 CONTINUE
            return HookAction.CONTINUE

        # 5) 提取命令后的剩余文本
        # 从原文中跳过命令名部分
        # 因为 shlex 可能改变 token 内容（去掉引号），这里用原文尝试匹配
        rest_text = text
        compare_text = text.lower() if self.ignore_case else text
        compare_name = matched_name.lower() if self.ignore_case else matched_name
        if compare_text.startswith(compare_name):
            rest_text = text[len(matched_name) :].strip()
        else:
            rest_text = ""

        # 6) 构建 binding stream
        stream = build_binding_stream(segments, rest_text)

        # 7) 绑定参数
        kwargs = bind_params(spec, stream)
        if kwargs is None:
            # 必选参数缺失 → WARNING + 回复 usage + SKIP
            LOG.warning(
                "命令 %s 必选参数未绑定 (handler=%s)",
                matched_name,
                func.__name__,
            )
            await reply_usage(ctx, self.names, spec)
            return HookAction.SKIP

        ctx.kwargs.update(kwargs)
        return HookAction.CONTINUE

    def _get_param_spec(self, func) -> Optional[_ParamSpec]:
        """解析并缓存 handler 的参数规格"""
        func_id = id(func)
        if func_id in self._sig_cache:
            return self._sig_cache[func_id]

        spec = get_param_spec(func)
        self._sig_cache[func_id] = spec
        return spec

    def __repr__(self) -> str:
        return f"<CommandHook(names={self.names!r}, ignore_case={self.ignore_case})>"



"""QQ 平台专用事件实体"""

from .message import MessageEvent, PrivateMessageEvent, GroupMessageEvent
from .notice import NoticeEvent, GroupIncreaseEvent
from .request import RequestEvent, FriendRequestEvent, GroupRequestEvent
from .meta import MetaEvent
from .factory import create_qq_entity

# 自动注册 QQ 平台工厂和 secondary keys 到通用工厂
from ncatbot.event.common.factory import (
    register_platform_factory as _register,
    register_platform_secondary_keys as _register_keys,
)

_register("qq", create_qq_entity)
_register_keys(
    "qq",
    {
        "message": "message_type",
        "message_sent": "message_type",
        "notice": "notice_type",
        "request": "request_type",
        "meta_event": "meta_event_type",
    },
)
del _register, _register_keys

__all__ = [
    "MessageEvent",
    "PrivateMessageEvent",
    "GroupMessageEvent",
    "NoticeEvent",
    "GroupIncreaseEvent",
    "RequestEvent",
    "FriendRequestEvent",
    "GroupRequestEvent",
    "MetaEvent",
    "create_qq_entity",
]



"""
插件加载器

组合 Indexer + Resolver + Importer，管理插件的完整生命周期。
通过 asyncio.create_task 订阅 FileWatcher 回调实现热重载。
"""

import asyncio
from pathlib import Path
from typing import Callable, Dict, List, Optional, Type, TYPE_CHECKING

from ncatbot.utils import get_log

from ..base import BasePlugin
from ..manifest import PluginManifest
from .indexer import PluginIndexer
from .resolver import DependencyResolver
from .importer import ModuleImporter
from .pip_helper import check_requirements, install_packages, format_missing_report

if TYPE_CHECKING:
    from ncatbot.core import HandlerDispatcher

LOG = get_log("PluginLoader")


class PluginLoader:
    """插件加载器

    典型用法::

        loader = PluginLoader(debug=True)
        await loader.load_all(Path("plugins"))

        # 热重载由 setup_hot_reload 自动配置
    """

    def __init__(self, *, debug: bool = False) -> None:
        self._debug = debug
        self._indexer = PluginIndexer()
        self._resolver = DependencyResolver()
        self._importer = ModuleImporter()
        self.plugins: Dict[str, BasePlugin] = {}

        # 热重载相关
        self._reload_queue: asyncio.Queue[str] = asyncio.Queue()
        self._reload_task: Optional[asyncio.Task] = None
        self._event_loop: Optional[asyncio.AbstractEventLoop] = None

        # 插件实例化时的注入回调（由外部设置）
        self._on_plugin_init: Optional[Callable[[BasePlugin, PluginManifest], None]] = (
            None
        )

        # Handler 分发器（由 BotClient 注入）
        self._handler_dispatcher: Optional["HandlerDispatcher"] = None

    # ------------------------------------------------------------------
    # 批量加载
    # ------------------------------------------------------------------

    async def load_all(self, plugin_dir: Path) -> List[str]:
        """扫描目录并按依赖顺序加载所有插件。

        Returns:
            成功加载的插件名列表
        """
        # 将插件根目录加入 sys.path（使跨插件导入可用）
        self._importer.add_plugin_root(plugin_dir)

        manifests = self._indexer.scan(plugin_dir)
        if not manifests:
            LOG.info("未在 %s 中发现插件", plugin_dir)
            return []

        # 依赖解析
        load_order = self._resolver.resolve(manifests)
        self._resolver.validate_versions(manifests)

        # pip 依赖检查
        skip_plugins = await self._check_pip_deps_batch(manifests)

        # 按序加载
        loaded = []
        for name in load_order:
            if name in skip_plugins:
                LOG.warning("跳过插件 %s（pip 依赖未满足）", name)
                continue
            plugin = await self.load_plugin(name)
            if plugin is not None:
                loaded.append(name)

        LOG.info("已加载 %d/%d 个插件", len(loaded), len(manifests))
        return loaded

    async def load_selected(
        self,
        plugin_dir: Path,
        names: List[str],
        *,
        skip_pip: bool = True,
    ) -> List[str]:
        """只加载指定插件及其传递依赖。

        Args:
            plugin_dir: 插件根目录
            names: 需要加载的目标插件名列表
            skip_pip: 是否跳过 pip 依赖检查（测试环境默认跳过）

        Returns:
            成功加载的插件名列表
        """
        self._importer.add_plugin_root(plugin_dir)

        manifests = self._indexer.scan(plugin_dir)
        if not manifests:
            LOG.info("未在 %s 中发现插件", plugin_dir)
            return []

        # 解析目标插件及其传递依赖的加载顺序
        load_order = self._resolver.resolve_subset(manifests, names)
        # 只对子集做版本验证
        subset = {n: manifests[n] for n in load_order}
        self._resolver.validate_versions(subset)

        # pip 依赖检查
        skip_plugins: set = set()
        if not skip_pip:
            skip_plugins = await self._check_pip_deps_batch(subset)

        loaded = []
        for name in load_order:
            if name in skip_plugins:
                LOG.warning("跳过插件 %s（pip 依赖未满足）", name)
                continue
            plugin = await self.load_plugin(name)
            if plugin is not None:
                loaded.append(name)

        LOG.info("已选择性加载 %d/%d 个插件", len(loaded), len(load_order))
        return loaded

    # ------------------------------------------------------------------
    # 单个插件操作
    # ------------------------------------------------------------------

    def set_handler_dispatcher(self, dispatcher: "HandlerDispatcher") -> None:
        """注入 HandlerDispatcher 实例（由 BotClient 调用）。"""
        self._handler_dispatcher = dispatcher

    async def load_plugin(self, name: str) -> Optional[BasePlugin]:
        """加载单个插件（必须已索引）。"""
        from ncatbot.core import (
            set_current_plugin,
        )
        from ncatbot.core.registry.context import (
            _current_plugin_ctx,
        )
        from ncatbot.core import flush_pending, clear_pending

        manifest = self._indexer.get(name)
        if manifest is None:
            LOG.error("插件 %s 未索引，无法加载", name)
            return None

        # 单插件加载时也检查 pip 依赖
        if manifest.pip_dependencies:
            if not await self._ensure_pip_deps(manifest):
                return None

        try:
            # ContextVar 包裹模块加载，确保装饰器能读取到当前插件名
            token = set_current_plugin(name)
            try:
                module = self._importer.load_module(manifest)
            finally:
                _current_plugin_ctx.reset(token)

            plugin_class = self._importer.find_plugin_class(
                module, manifest.entry_class
            )
            plugin = self._instantiate(plugin_class, manifest)

            # 框架加载
            await plugin.__onload__()

            # 将装饰器收集的 handler 注册到 HandlerDispatcher
            if self._handler_dispatcher is not None:
                flush_pending(self._handler_dispatcher, name, plugin_instance=plugin)

            self.plugins[name] = plugin
            LOG.info("插件加载成功: %s v%s", name, manifest.version)
            return plugin

        except Exception as e:
            # 加载失败时清理残留的 pending handlers
            clear_pending(name)
            LOG.error("加载插件 %s 失败: %s", name, e)
            return None

    async def unload_plugin(self, name: str) -> bool:
        """卸载单个插件。"""
        from ncatbot.core import clear_pending

        plugin = self.plugins.get(name)
        if plugin is None:
            LOG.warning("插件 %s 未加载", name)
            return False

        manifest = self._indexer.get(name)
        try:
            await plugin.__unload__()
        except Exception as e:
            LOG.error("插件 %s __unload__ 异常: %s", name, e)

        # 从 HandlerDispatcher 移除该插件的所有 handler
        if self._handler_dispatcher is not None:
            self._handler_dispatcher.revoke_plugin(name)

        # 清理可能残留的 pending handlers
        clear_pending(name)

        if manifest is not None:
            self._importer.unload_module(manifest)

        del self.plugins[name]
        LOG.info("插件已卸载: %s", name)
        return True

    async def reload_plugin(self, name: str) -> bool:
        """重载单个插件（卸载 → 重索引 → 加载）。"""
        manifest = self._indexer.get(name)
        if manifest is None:
            LOG.error("无法重载未索引的插件: %s", name)
            return False

        await self.unload_plugin(name)
        await asyncio.sleep(0.02)  # 等待清理完成

        # 重新读取 manifest（支持清单变更）
        new_manifest = self._indexer.rescan_plugin(name)
        if new_manifest is None:
            LOG.error("重载插件 %s 时重索引失败", name)
            return False

        result = await self.load_plugin(name)
        if result is not None:
            LOG.info("插件 %s 重载成功", name)
            return True
        return False

    async def unload_all(self) -> None:
        """卸载所有插件。"""
        await asyncio.gather(*(self.unload_plugin(name) for name in list(self.plugins)))

    # ------------------------------------------------------------------
    # 查询
    # ------------------------------------------------------------------

    def get_plugin(self, name: str) -> Optional[BasePlugin]:
        return self.plugins.get(name)

    def list_plugins(self) -> List[str]:
        return list(self.plugins.keys())

    def get_plugin_name_by_folder(self, folder_name: str) -> Optional[str]:
        """根据文件夹名查找插件名。"""
        manifest = self._indexer.get_by_folder(folder_name)
        return manifest.name if manifest else None

    # ------------------------------------------------------------------
    # 热重载
    # ------------------------------------------------------------------

    def setup_hot_reload(
        self, file_watcher, *, loop: Optional[asyncio.AbstractEventLoop] = None
    ) -> None:
        """配置热重载。

        将 FileWatcherService 的回调连接到重载队列，并启动异步消费任务。

        Args:
            file_watcher: FileWatcherService 实例
            loop: 事件循环（默认使用当前 running loop）
        """
        self._event_loop = loop or asyncio.get_running_loop()

        def _on_change(plugin_folder: str):
            """FileWatcher 回调（在 watcher 线程中执行）"""
            assert self._event_loop is not None
            self._event_loop.call_soon_threadsafe(
                self._reload_queue.put_nowait, plugin_folder
            )

        file_watcher.on_file_changed = _on_change
        self._reload_task = asyncio.ensure_future(
            self._reload_consumer(), loop=self._event_loop
        )
        LOG.info("热重载已启动")

    async def stop_hot_reload(self) -> None:
        """停止热重载消费任务。"""
        if self._reload_task and not self._reload_task.done():
            self._reload_task.cancel()
            try:
                await self._reload_task
            except asyncio.CancelledError:
                pass
            self._reload_task = None
            LOG.info("热重载已停止")

    async def _reload_consumer(self) -> None:
        """异步任务：从队列中消费文件变更事件并执行重载。"""
        while True:
            folder_name = await self._reload_queue.get()
            plugin_name = self.get_plugin_name_by_folder(folder_name)
            if plugin_name is None:
                LOG.debug("未知插件目录: %s，跳过重载", folder_name)
                continue

            if plugin_name not in self.plugins:
                LOG.debug("插件 %s 未加载，跳过重载", plugin_name)
                continue

            LOG.info("热重载触发: %s (文件夹: %s)", plugin_name, folder_name)
            try:
                await self.reload_plugin(plugin_name)
            except Exception as e:
                LOG.error("热重载失败 [%s]: %s", plugin_name, e)

    # ------------------------------------------------------------------
    # 内置插件
    # ------------------------------------------------------------------

    async def load_builtin_plugins(self) -> None:
        """加载内置插件。"""
        from ...plugin.builtin import BUILTIN_PLUGINS

        for plugin_class in BUILTIN_PLUGINS:
            name = plugin_class.name
            if name in self.plugins:
                LOG.warning("内置插件 %s 已加载，跳过", name)
                continue

            manifest = PluginManifest(
                name=name,
                version=plugin_class.version,
                main="__builtin__",
                author=getattr(plugin_class, "author", "NcatBot"),
                description=getattr(plugin_class, "description", ""),
                plugin_dir=Path("data") / name,
                folder_name=name,
            )

            try:
                plugin = self._instantiate(plugin_class, manifest)
                await plugin.__onload__()
                self.plugins[name] = plugin
                LOG.info("内置插件加载成功: %s v%s", name, manifest.version)
            except Exception as e:
                LOG.error("加载内置插件 %s 失败: %s", name, e)

    # ------------------------------------------------------------------
    # 内部方法
    # ------------------------------------------------------------------

    async def _check_pip_deps_batch(self, manifests: Dict[str, PluginManifest]) -> set:
        """批量检查所有插件的 pip 依赖，返回应跳过的插件名集合。"""
        from ncatbot.utils import async_confirm
        from ncatbot.utils import get_config_manager

        config = get_config_manager()
        if not config.plugin.auto_install_pip_deps:
            # 配置禁用自动安装，仅输出警告
            skip = set()
            for name, manifest in manifests.items():
                if manifest.pip_dependencies:
                    _, missing = check_requirements(manifest.pip_dependencies)
                    if missing:
                        LOG.warning("%s", format_missing_report(name, missing))
                        skip.add(name)
            return skip

        # 合并所有插件的 pip 依赖（去重，同名取第一个出现的约束）
        all_deps: Dict[str, str] = {}
        dep_to_plugins: Dict[str, List[str]] = {}
        for name, manifest in manifests.items():
            for pkg, constraint in manifest.pip_dependencies.items():
                if pkg not in all_deps:
                    all_deps[pkg] = constraint
                dep_to_plugins.setdefault(pkg, []).append(name)

        if not all_deps:
            return set()

        _, missing = check_requirements(all_deps)
        if not missing:
            return set()

        # 构建报告
        report_lines = ["以下插件需要额外安装 pip 依赖:"]
        for req in missing:
            pkg_name = (
                req.split(">")[0]
                .split("<")[0]
                .split("=")[0]
                .split("!")[0]
                .split("~")[0]
                .strip()
            )
            plugins = dep_to_plugins.get(pkg_name, ["?"])
            report_lines.append(f"  - {req} (被 {', '.join(plugins)} 使用)")
        report = "\n".join(report_lines)
        LOG.info("%s", report)

        # 询问用户
        approved = await async_confirm(
            f"是否安装以上 {len(missing)} 个依赖?", default=True
        )
        if not approved:
            # 找出哪些插件受影响
            missing_names = {
                req.split(">")[0]
                .split("<")[0]
                .split("=")[0]
                .split("!")[0]
                .split("~")[0]
                .strip()
                for req in missing
            }
            skip = set()
            for name, manifest in manifests.items():
                if missing_names & set(manifest.pip_dependencies.keys()):
                    skip.add(name)
            LOG.info("用户拒绝安装，跳过插件: %s", skip)
            return skip

        # 执行安装
        if install_packages(missing):
            return set()

        # 安装失败，重新检查哪些还缺
        _, still_missing = check_requirements(all_deps)
        if not still_missing:
            return set()

        still_missing_names = {
            req.split(">")[0]
            .split("<")[0]
            .split("=")[0]
            .split("!")[0]
            .split("~")[0]
            .strip()
            for req in still_missing
        }
        skip = set()
        for name, manifest in manifests.items():
            if still_missing_names & set(manifest.pip_dependencies.keys()):
                skip.add(name)
        LOG.warning("部分依赖安装失败，跳过插件: %s", skip)
        return skip

    async def _ensure_pip_deps(self, manifest: PluginManifest) -> bool:
        """检查并安装单个插件的 pip 依赖。返回 True 表示满足/已安装。"""
        from ncatbot.utils import async_confirm
        from ncatbot.utils import get_config_manager

        _, missing = check_requirements(manifest.pip_dependencies)
        if not missing:
            return True

        LOG.info("%s", format_missing_report(manifest.name, missing))

        config = get_config_manager()
        if not config.plugin.auto_install_pip_deps:
            LOG.warning("auto_install_pip_deps 已禁用，跳过插件 %s", manifest.name)
            return False

        approved = await async_confirm(
            f"插件 {manifest.name} 需要安装 {len(missing)} 个 pip 依赖，是否安装?",
            default=True,
        )
        if not approved:
            LOG.info("用户拒绝安装，跳过插件 %s", manifest.name)
            return False

        if not install_packages(missing):
            LOG.error("依赖安装失败，跳过插件 %s", manifest.name)
            return False

        return True

    def _instantiate(
        self, plugin_class: Type[BasePlugin], manifest: PluginManifest
    ) -> BasePlugin:
        """实例化插件并注入运行时属性。"""
        plugin = plugin_class()

        # 注入插件元数据（manifest 优先于类属性）
        plugin.name = manifest.name
        plugin.version = manifest.version
        if manifest.author:
            plugin.author = manifest.author
        if manifest.description:
            plugin.description = manifest.description

        # 注入框架属性
        plugin._manifest = manifest
        plugin._debug = self._debug
        plugin.workspace = Path("data") / manifest.name
        plugin.config = {}
        plugin.data = {}

        # 外部注入回调
        if self._on_plugin_init:
            self._on_plugin_init(plugin, manifest)

        return plugin



"""MiscAPI — 杂项工具 API（下载、HTTP 请求、代理检查）

通过 ``api.misc`` 访问，提供与平台无关的 HTTP/下载/代理工具::

    # 下载文件
    path = await api.misc.download_to_file(url, "./downloads")

    # 下载到内存
    data = await api.misc.download_to_bytes(url)

    # GET 请求
    body = await api.misc.http_get("https://api.example.com/data")

    # 检查代理
    ok = await api.misc.is_proxy_valid()
"""

from __future__ import annotations

from pathlib import Path
from typing import Optional, Union, TYPE_CHECKING

from ncatbot.utils import get_log
from ncatbot.utils.network import (
    async_check_proxy,
    async_download_to_bytes,
    async_download_to_file,
    async_http_get,
)

if TYPE_CHECKING:
    from ncatbot.utils.config.models import Config

LOG = get_log("MiscAPI")


class MiscAPI:
    """杂项工具 API — 下载、HTTP 请求、代理检查。

    所有带 ``proxy`` 参数的方法：显式传值时使用传入的代理；
    省略 / ``None`` 时自动使用主配置 ``http_proxy``；
    配置也为空则直连。
    """

    def __init__(self, config: "Config") -> None:
        self._config = config

    def get_proxy(self) -> Optional[str]:
        """返回当前主配置中的 ``http_proxy``，无配置时返回 ``None``。"""
        return self._config.http_proxy or None

    def _resolve_proxy(self, proxy: Optional[str]) -> Optional[str]:
        """解析代理：显式传值优先，否则取配置。"""
        if proxy is not None:
            return proxy
        return self.get_proxy()

    async def download_to_file(
        self,
        url: str,
        dest_dir: Union[str, Path],
        *,
        filename: Optional[str] = None,
        proxy: Optional[str] = None,
    ) -> Path:
        """下载 URL 到目录，返回文件路径。

        Parameters
        ----------
        url : str
            下载地址。
        dest_dir : str | Path
            目标目录。
        filename : str, optional
            文件名，省略则从 URL 推断。
        proxy : str, optional
            代理地址，省略则使用主配置 ``http_proxy``。
        """
        resolved = self._resolve_proxy(proxy)
        return await async_download_to_file(
            url, dest_dir, filename=filename, proxy=resolved
        )

    async def download_to_bytes(
        self,
        url: str,
        *,
        proxy: Optional[str] = None,
    ) -> bytes:
        """下载 URL 到内存，返回原始字节。

        Parameters
        ----------
        url : str
            下载地址。
        proxy : str, optional
            代理地址，省略则使用主配置 ``http_proxy``。
        """
        resolved = self._resolve_proxy(proxy)
        return await async_download_to_bytes(url, proxy=resolved)

    async def http_get(
        self,
        url: str,
        *,
        headers: Optional[dict] = None,
        proxy: Optional[str] = None,
        timeout: float = 10.0,
    ) -> bytes:
        """异步 GET 请求，返回响应体字节。

        Parameters
        ----------
        url : str
            请求地址。
        headers : dict, optional
            额外请求头。
        proxy : str, optional
            代理地址，省略则使用主配置 ``http_proxy``。
        timeout : float
            超时秒数，默认 10。
        """
        resolved = self._resolve_proxy(proxy)
        return await async_http_get(
            url, headers=headers, proxy=resolved, timeout=timeout
        )

    async def is_proxy_valid(self, proxy: Optional[str] = None) -> bool:
        """检查代理是否可用。

        Parameters
        ----------
        proxy : str, optional
            要检查的代理地址。省略则检查主配置 ``http_proxy``。
            如果配置也为空，直接返回 ``False``。

        Returns
        -------
        bool
        """
        resolved = self._resolve_proxy(proxy)
        if not resolved:
            return False
        return await async_check_proxy(resolved)



"""
平台操作抽象层

使用工厂模式统一 Windows/Linux 平台的 NapCat 目录、启动、停止操作。
安装逻辑已提取到 installer.py。
"""

import json
import platform
import subprocess
import sys
import time
from abc import ABC, abstractmethod
from pathlib import Path
from typing import Optional

if sys.platform == "win32":
    import ctypes
    import winreg

from ncatbot.utils import get_log
from ncatbot.utils import confirm
from ..constants import LINUX_NAPCAT_DIR, WINDOWS_NAPCAT_DIR

LOG = get_log("NapCatPlatform")


class UnsupportedPlatformError(Exception):
    """不支持的操作系统"""

    def __init__(self, system: Optional[str] = None):
        system = system or platform.system()
        super().__init__(f"不支持的操作系统: {system}")


class PlatformOps(ABC):
    """平台操作抽象基类"""

    @staticmethod
    def create() -> "PlatformOps":
        system = platform.system()
        if system == "Windows":
            return WindowsOps()
        elif system == "Linux":
            return LinuxOps()
        raise UnsupportedPlatformError(system)

    @property
    @abstractmethod
    def napcat_dir(self) -> Path: ...

    @property
    def config_dir(self) -> Path:
        return self.napcat_dir / "config"

    @abstractmethod
    def is_napcat_running(self, uin: Optional[str] = None) -> bool: ...

    @abstractmethod
    def start_napcat(self, uin: str) -> None: ...

    @abstractmethod
    def stop_napcat(self) -> None: ...

    def is_napcat_installed(self) -> bool:
        return self.napcat_dir.exists()

    def get_installed_version(self) -> Optional[str]:
        package_json = self.napcat_dir / "package.json"
        if not package_json.exists():
            return None
        try:
            with open(package_json, "r", encoding="utf-8") as f:
                return json.load(f)["version"]
        except (json.JSONDecodeError, KeyError):
            return None

    @staticmethod
    def _confirm_action(prompt: str) -> bool:
        return confirm(prompt, default=False)


class WindowsOps(PlatformOps):
    """Windows 平台操作"""

    @property
    def napcat_dir(self) -> Path:
        return Path(WINDOWS_NAPCAT_DIR)

    def is_napcat_running(self, uin: Optional[str] = None) -> bool:
        try:
            result = subprocess.run(
                ["tasklist", "/FI", "IMAGENAME eq QQ.exe", "/FO", "CSV", "/NH"],
                capture_output=True,
                text=True,
                timeout=5,
            )
            return "QQ.exe" in result.stdout
        except Exception:
            return False

    def start_napcat(self, uin: str) -> None:
        launcher = self._get_launcher_name()
        launcher_path = self.napcat_dir.resolve() / launcher

        if not launcher_path.exists():
            raise FileNotFoundError(f"找不到启动文件: {launcher_path}")

        LOG.info(f"正在启动 QQ, 启动器路径: {launcher_path}")

        # 通过 ShellExecuteW + runas 进行 UAC 提权启动 launcher.bat
        # launcher.bat 在独立的管理员进程中运行, NcatBot 退出不影响 NapCat
        ret = ctypes.windll.shell32.ShellExecuteW(
            None,  # hwnd
            "runas",  # lpOperation: 请求管理员权限
            "cmd.exe",  # lpFile
            f'/c cd /d "{self.napcat_dir.resolve()}" && "{launcher_path}" {uin}',
            None,  # lpDirectory
            1,  # nShowCmd: SW_SHOWNORMAL, 显示 NapCat 运行状态窗口
        )
        # ShellExecuteW 返回值 > 32 表示成功
        if ret <= 32:
            raise RuntimeError(
                f"UAC 提权启动失败 (错误码={ret}), "
                f"请以管理员身份运行 NcatBot 或手动执行: {launcher_path} {uin}"
            )

    # # ==================== 备用: 直接调用模式 (无需管理员) ====================
    # def _start_napcat_direct(self, uin: str) -> None:
    #     """直接调用 NapCatWinBootMain.exe, 绕过 launcher.bat 的管理员检查"""
    #     napcat_dir = self.napcat_dir.resolve()
    #
    #     launcher_exe = napcat_dir / "NapCatWinBootMain.exe"
    #     inject_dll = napcat_dir / "NapCatWinBootHook.dll"
    #     napcat_mjs = napcat_dir / "napcat.mjs"
    #     load_js = napcat_dir / "loadNapCat.js"
    #
    #     for path in [launcher_exe, inject_dll, napcat_mjs]:
    #         if not path.exists():
    #             raise FileNotFoundError(f"找不到 NapCat 文件: {path}")
    #
    #     qq_path = self._find_qq_path()
    #     if not qq_path.exists():
    #         raise FileNotFoundError(f"找不到 QQ.exe: {qq_path}")
    #
    #     napcat_main_posix = napcat_mjs.as_posix()
    #     load_js.write_text(
    #         f'(async () => {{await import("file:///{napcat_main_posix}")}})()',
    #         encoding="utf-8",
    #     )
    #
    #     env = os.environ.copy()
    #     env["NAPCAT_PATCH_PACKAGE"] = str(napcat_dir / "qqnt.json")
    #     env["NAPCAT_LOAD_PATH"] = str(load_js)
    #     env["NAPCAT_INJECT_PATH"] = str(inject_dll)
    #     env["NAPCAT_LAUNCHER_PATH"] = str(launcher_exe)
    #     env["NAPCAT_MAIN_PATH"] = str(napcat_mjs)
    #
    #     LOG.info(f"正在启动 QQ, 注入器: {launcher_exe}")
    #     process = subprocess.Popen(
    #         [str(launcher_exe), str(qq_path), str(inject_dll), str(uin)],
    #         cwd=str(napcat_dir),
    #         env=env,
    #         stdout=subprocess.PIPE,
    #         stderr=subprocess.PIPE,
    #     )
    #
    #     try:
    #         process.wait(timeout=3)
    #         stdout = process.stdout.read().decode(errors="ignore").strip()
    #         stderr = process.stderr.read().decode(errors="ignore").strip()
    #         if process.returncode != 0:
    #             detail = stderr or stdout or "无输出"
    #             raise RuntimeError(
    #                 f"NapCat 启动器异常退出 (code={process.returncode}): {detail}"
    #             )
    #     except subprocess.TimeoutExpired:
    #         LOG.debug("NapCat 启动器进程运行中")

    def stop_napcat(self) -> None:
        LOG.warning("Windows 下不支持按进程名停止 NapCat (会误杀普通 QQ 客户端)")

    def _get_launcher_name(self) -> str:
        platform_info = platform.platform()

        try:
            edition = platform.win32_edition()
            is_server = "Server" in edition
        except AttributeError:
            is_server = "Server" in platform_info

        if is_server:
            if any(ver in platform_info for ver in ["2016", "2019", "2022"]):
                LOG.info("当前操作系统: Windows Server (旧版本)")
                return "launcher-win10.bat"
            elif "2025" in platform_info:
                LOG.info("当前操作系统: Windows Server 2025")
                return "launcher.bat"
            LOG.warning("不支持的 Windows Server 版本，按 Windows 10 内核启动")
            return "launcher-win10.bat"

        release = platform.release()
        if release == "11":
            LOG.info("当前操作系统: Windows 11")
            return "launcher.bat"

        LOG.info("当前操作系统: Windows 10")
        return "launcher-win10.bat"

    @staticmethod
    def _find_qq_path() -> Path:
        """从注册表查找 QQ 安装路径"""
        reg_key = r"SOFTWARE\WOW6432Node\Microsoft\Windows\CurrentVersion\Uninstall\QQ"
        try:
            with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, reg_key) as key:
                uninstall_str, _ = winreg.QueryValueEx(key, "UninstallString")
                return Path(uninstall_str).parent / "QQ.exe"
        except OSError:
            raise FileNotFoundError(
                "无法从注册表找到 QQ 安装路径, 请确认 QQ 已正确安装"
            )


class LinuxOps(PlatformOps):
    """Linux 平台操作"""

    @property
    def napcat_dir(self) -> Path:
        target = Path(LINUX_NAPCAT_DIR)
        if target.exists():
            return target
        return Path.home() / "Napcat/opt/QQ/resources/app/app_launcher/napcat"

    def is_napcat_running(self, uin: Optional[str] = None) -> bool:
        try:
            process = subprocess.Popen(
                ["napcat", "status"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
            process.wait()
            stdout = process.stdout
            if stdout is None:
                return False
            output = stdout.read().decode(encoding="utf-8")

            if uin is None:
                return "PID" in output
            return str(uin) in output
        except FileNotFoundError:
            LOG.warning("napcat CLI 未安装或不在 PATH 中")
            return False

    def start_napcat(self, uin: str) -> None:
        if self.is_napcat_running(uin):
            LOG.info("NapCat 已启动")
            return

        if self.is_napcat_running():
            LOG.warning("NapCat 正在运行, 但运行的不是该 QQ 号")
            if confirm("强制结束当前 NapCat 进程并继续?", default=False):
                self.stop_napcat()
            else:
                raise RuntimeError("NapCat 正在运行, 但运行的不是该 QQ 号")

        LOG.info("正在启动 NapCat 服务")
        process = subprocess.Popen(
            ["napcat", "start", uin],
            stdout=subprocess.PIPE,
        )
        process.wait()

        if process.returncode != 0:
            raise FileNotFoundError(
                "napcat CLI 启动失败, 请确认 napcat CLI 已正确安装到 /usr/local/bin/napcat"
            )

        if not self.is_napcat_running(uin):
            raise RuntimeError("napcat 启动失败")

        time.sleep(0.5)
        LOG.info("napcat 启动成功")

    def stop_napcat(self) -> None:
        try:
            process = subprocess.Popen(
                ["napcat", "stop"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
            process.wait()
            if process.returncode != 0:
                raise RuntimeError("停止 napcat 失败")
            LOG.info("已成功停止 napcat")
        except FileNotFoundError:
            LOG.error("napcat CLI 未安装, 无法停止 NapCat")
            raise
        except Exception as e:
            LOG.error(f"停止 napcat 失败: {e}")
            raise


class MacOps(PlatformOps):
    """macOS 平台操作"""

    @property
    def napcat_dir(self) -> Path:
        raise UnsupportedPlatformError("macOS 平台暂不支持 NcatBot")

    def is_napcat_running(self, uin: Optional[str] = None) -> bool:
        raise UnsupportedPlatformError("macOS 平台暂不支持 NcatBot")

    def start_napcat(self, uin: str) -> None:
        raise UnsupportedPlatformError("macOS 平台暂不支持 NcatBot")

    def stop_napcat(self) -> None:
        raise UnsupportedPlatformError("macOS 平台暂不支持 NcatBot")



"""动态页数据源 — 轮询动态主页，一次性监听多个订阅 UP 主的新动态

通过 bilibili_api.dynamic.get_dynamic_page_info 拉取动态主页，
按订阅 UID 集合过滤，基于每个 UP 主的最新时间戳增量检测新动态。
"""

from __future__ import annotations

import asyncio
from typing import Any, Awaitable, Callable, Dict, Optional, Set

from ncatbot.utils import get_log

from .base import BaseSource

LOG = get_log("DynamicPageSource")

# 默认轮询间隔 3 分钟
_DEFAULT_POLL_INTERVAL = 180.0


class DynamicPageSource(BaseSource):
    source_type = "dynamic"

    def __init__(
        self,
        credential: Any,
        callback: Callable[[str, dict], Awaitable[None]],
        *,
        poll_interval: float = _DEFAULT_POLL_INTERVAL,
    ) -> None:
        super().__init__(callback)
        self.source_id = "dynamic_page"
        self._credential = credential
        self._poll_interval = poll_interval
        # 订阅的 UP 主 UID 集合
        self._watched_uids: Set[int] = set()
        # 每个 UP 主的最新已知动态时间戳
        self._last_ts: Dict[int, int] = {}
        self._task: Optional[asyncio.Task] = None
        self._stop_event = asyncio.Event()

    def add_uid(self, uid: int) -> None:
        """添加一个订阅 UID"""
        self._watched_uids.add(uid)

    def remove_uid(self, uid: int) -> None:
        """移除一个订阅 UID"""
        self._watched_uids.discard(uid)
        self._last_ts.pop(uid, None)

    def has_uid(self, uid: int) -> bool:
        return uid in self._watched_uids

    @property
    def empty(self) -> bool:
        return len(self._watched_uids) == 0

    async def start(self) -> None:
        if self._running:
            return
        self._stop_event.clear()
        self._running = True
        self._task = asyncio.create_task(self._poll_loop(), name="dynamic_page_source")
        LOG.info(
            "动态页源已启动 (轮询间隔 %.0fs, 订阅 %d 个 UID)",
            self._poll_interval,
            len(self._watched_uids),
        )

    # ---- 静态工具 ----

    @staticmethod
    def _extract_uid(item: dict) -> int:
        """从动态 item 中提取作者 UID"""
        modules = item.get("modules") or {}
        author = modules.get("module_author") or {}
        return int(author.get("mid", 0))

    @staticmethod
    def _extract_pub_ts(item: dict) -> int:
        """提取动态发布时间戳"""
        modules = item.get("modules") or {}
        author = modules.get("module_author") or {}
        return int(author.get("pub_ts", 0))

    # ---- 轮询 ----

    async def _fetch_page(self) -> list:
        """拉取动态主页第一页"""
        from bilibili_api.dynamic import get_dynamic_page_info

        resp = await get_dynamic_page_info(credential=self._credential)
        return resp.get("items") or []

    async def _poll_loop(self) -> None:
        # 初次拉取，建立时间戳基线
        try:
            items = await self._fetch_page()
            self._init_baselines(items)
        except Exception:
            LOG.warning("动态页源初次拉取失败", exc_info=True)

        while not self._stop_event.is_set():
            try:
                await asyncio.wait_for(
                    self._stop_event.wait(), timeout=self._poll_interval
                )
                break
            except asyncio.TimeoutError:
                pass

            try:
                items = await self._fetch_page()
                await self._detect_new_dynamics(items)
            except asyncio.CancelledError:
                break
            except Exception:
                LOG.debug("动态页源轮询异常", exc_info=True)

        self._running = False

    def _init_baselines(self, items: list) -> None:
        """用首次拉取的结果初始化每个订阅 UID 的时间戳基线"""
        for item in items:
            uid = self._extract_uid(item)
            if uid not in self._watched_uids:
                continue
            ts = self._extract_pub_ts(item)
            # 只记录更大的时间戳（同一 UID 可能有多条）
            if ts > self._last_ts.get(uid, 0):
                self._last_ts[uid] = ts
        LOG.debug("动态页源基线: %s", self._last_ts)

    async def _detect_new_dynamics(self, items: list) -> None:
        """检测并推送新动态"""
        # 按 UID 分组，收集每个订阅 UP 主的动态
        uid_items: Dict[int, list] = {}
        for item in items:
            uid = self._extract_uid(item)
            if uid not in self._watched_uids:
                continue
            uid_items.setdefault(uid, []).append(item)

        for uid, dyn_items in uid_items.items():
            old_ts = self._last_ts.get(uid, 0)
            # 过滤出新动态（时间戳严格大于已知最大值）
            new_items = [it for it in dyn_items if self._extract_pub_ts(it) > old_ts]
            if not new_items:
                continue

            # 按时间戳升序推送，确保先发的先推
            new_items.sort(key=self._extract_pub_ts)
            for item in new_items:
                ts = self._extract_pub_ts(item)
                raw = {
                    "source": "dynamic",
                    "uid": uid,
                    "status": "new",
                    "dynamic": item,
                }
                await self._callback("dynamic", raw)
                # 更新时间戳
                if ts > self._last_ts.get(uid, 0):
                    self._last_ts[uid] = ts

    async def stop(self) -> None:
        self._stop_event.set()
        if self._task is not None and not self._task.done():
            self._task.cancel()
            try:
                await self._task
            except (asyncio.CancelledError, Exception):
                pass
        self._running = False
        self._task = None
        LOG.debug("动态页源已停止")



"""
事件工厂 — 快速构建测试用 BaseEventData 实例

所有 factory 方法产出经过 model_validate 的合法数据模型，
可直接喂入 MockAdapter.inject_event()。
"""

from __future__ import annotations

import time
from itertools import count
from typing import Any, Dict, List, Optional

from ncatbot.types.qq import (
    GroupMessageEventData,
    PrivateMessageEventData,
    FriendRequestEventData,
    GroupRequestEventData,
    GroupIncreaseNoticeEventData,
    GroupDecreaseNoticeEventData,
    GroupBanNoticeEventData,
    PokeNotifyEventData,
)

_msg_id_counter = count(1)
_SELF_ID = "10001"


def _next_msg_id() -> str:
    return str(next(_msg_id_counter))


def _now() -> int:
    return int(time.time())


def _text_segments(text: str) -> List[Dict[str, Any]]:
    return [{"type": "text", "data": {"text": text}}]


def group_message(
    text: str = "hello",
    *,
    group_id: str = "100200",
    user_id: str = "99999",
    nickname: str = "测试用户",
    message_id: Optional[str] = None,
    self_id: str = _SELF_ID,
    message: Optional[list] = None,
    raw_message: Optional[str] = None,
    sub_type: str = "normal",
    **extra: Any,
) -> GroupMessageEventData:
    """构造群消息事件"""
    data = {
        "time": _now(),
        "self_id": self_id,
        "platform": "qq",
        "post_type": "message",
        "message_type": "group",
        "sub_type": sub_type,
        "message_id": message_id or _next_msg_id(),
        "group_id": group_id,
        "user_id": user_id,
        "message": message or _text_segments(text),
        "raw_message": raw_message or text,
        "font": 0,
        "sender": {
            "user_id": user_id,
            "nickname": nickname,
            "card": nickname,
            "role": "member",
        },
        **extra,
    }
    return GroupMessageEventData.model_validate(data)


def private_message(
    text: str = "hello",
    *,
    user_id: str = "99999",
    nickname: str = "测试用户",
    message_id: Optional[str] = None,
    self_id: str = _SELF_ID,
    message: Optional[list] = None,
    raw_message: Optional[str] = None,
    sub_type: str = "friend",
    **extra: Any,
) -> PrivateMessageEventData:
    """构造私聊消息事件"""
    data = {
        "time": _now(),
        "self_id": self_id,
        "platform": "qq",
        "post_type": "message",
        "message_type": "private",
        "sub_type": sub_type,
        "message_id": message_id or _next_msg_id(),
        "user_id": user_id,
        "message": message or _text_segments(text),
        "raw_message": raw_message or text,
        "font": 0,
        "sender": {
            "user_id": user_id,
            "nickname": nickname,
        },
        **extra,
    }
    return PrivateMessageEventData.model_validate(data)


def friend_request(
    user_id: str = "99999",
    comment: str = "请求加好友",
    flag: str = "flag_123",
    *,
    self_id: str = _SELF_ID,
    **extra: Any,
) -> FriendRequestEventData:
    """构造好友请求事件"""
    data = {
        "time": _now(),
        "self_id": self_id,
        "platform": "qq",
        "post_type": "request",
        "request_type": "friend",
        "user_id": user_id,
        "comment": comment,
        "flag": flag,
        **extra,
    }
    return FriendRequestEventData.model_validate(data)


def group_request(
    user_id: str = "99999",
    group_id: str = "100200",
    comment: str = "请求加群",
    flag: str = "flag_456",
    sub_type: str = "add",
    *,
    self_id: str = _SELF_ID,
    **extra: Any,
) -> GroupRequestEventData:
    """构造加群请求事件"""
    data = {
        "time": _now(),
        "self_id": self_id,
        "platform": "qq",
        "post_type": "request",
        "request_type": "group",
        "sub_type": sub_type,
        "group_id": group_id,
        "user_id": user_id,
        "comment": comment,
        "flag": flag,
        **extra,
    }
    return GroupRequestEventData.model_validate(data)


def group_increase(
    user_id: str = "99999",
    group_id: str = "100200",
    operator_id: str = "10001",
    sub_type: str = "approve",
    *,
    self_id: str = _SELF_ID,
    **extra: Any,
) -> GroupIncreaseNoticeEventData:
    """构造群成员增加通知"""
    data = {
        "time": _now(),
        "self_id": self_id,
        "platform": "qq",
        "post_type": "notice",
        "notice_type": "group_increase",
        "sub_type": sub_type,
        "group_id": group_id,
        "operator_id": operator_id,
        "user_id": user_id,
        **extra,
    }
    return GroupIncreaseNoticeEventData.model_validate(data)


def group_decrease(
    user_id: str = "99999",
    group_id: str = "100200",
    operator_id: str = "10001",
    sub_type: str = "kick",
    *,
    self_id: str = _SELF_ID,
    **extra: Any,
) -> GroupDecreaseNoticeEventData:
    """构造群成员减少通知"""
    data = {
        "time": _now(),
        "self_id": self_id,
        "platform": "qq",
        "post_type": "notice",
        "notice_type": "group_decrease",
        "sub_type": sub_type,
        "group_id": group_id,
        "operator_id": operator_id,
        "user_id": user_id,
        **extra,
    }
    return GroupDecreaseNoticeEventData.model_validate(data)


def group_ban(
    user_id: str = "99999",
    group_id: str = "100200",
    operator_id: str = "10001",
    duration: int = 600,
    sub_type: str = "ban",
    *,
    self_id: str = _SELF_ID,
    **extra: Any,
) -> GroupBanNoticeEventData:
    """构造群禁言通知"""
    data = {
        "time": _now(),
        "self_id": self_id,
        "platform": "qq",
        "post_type": "notice",
        "notice_type": "group_ban",
        "sub_type": sub_type,
        "group_id": group_id,
        "operator_id": operator_id,
        "user_id": user_id,
        "duration": duration,
        **extra,
    }
    return GroupBanNoticeEventData.model_validate(data)


def poke(
    user_id: str = "99999",
    target_id: str = "10001",
    group_id: str = "100200",
    *,
    self_id: str = _SELF_ID,
    **extra: Any,
) -> PokeNotifyEventData:
    """构造戳一戳通知"""
    data = {
        "time": _now(),
        "self_id": self_id,
        "platform": "qq",
        "post_type": "notice",
        "notice_type": "notify",
        "sub_type": "poke",
        "group_id": group_id,
        "user_id": user_id,
        "target_id": target_id,
        **extra,
    }
    return PokeNotifyEventData.model_validate(data)
